// FileTextureControl.cpp : implementation file
//

#include "stdafx.h"
#include "ClassExampleDlg.h"
#include "FileTextureControl.h"


// FileTextureControl dialog

IMPLEMENT_DYNAMIC(FileTextureControl, CDialog)
FileTextureControl::FileTextureControl(CWnd* pParent /*=NULL*/)
	: CDialog(FileTextureControl::IDD, pParent)
  , m_V1(NULL)
  , m_V2(NULL)
  , m_F1(NULL)
  , m_F2(NULL)
{
}

FileTextureControl::~FileTextureControl()
{
}

void FileTextureControl::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  DDX_Control(pDX, IDC_TEX_ROTATE,      m_TexRotate);
  DDX_Control(pDX, IDC_TEX_TRANSLATE_X, m_TexTransX);
  DDX_Control(pDX, IDC_TEX_TRANSLATE_Y, m_TexTransY);
  DDX_Control(pDX, IDC_TEX_SCALE_X,     m_TexScaleX);
  DDX_Control(pDX, IDC_TEX_SCALE_Y,     m_TexScaleY);
}


BEGIN_MESSAGE_MAP(FileTextureControl, CDialog)
  ON_BN_CLICKED(IDC_TEX_U, OnBnClickedTexU)
  ON_BN_CLICKED(IDC_TEX_V, OnBnClickedTexV)
  ON_BN_CLICKED(IDC_TEX_WRAP, OnBnClickedTexWrap)
  ON_BN_CLICKED(IDC_TEX_MIRROR, OnBnClickedTexMirror)
  ON_BN_CLICKED(IDC_TEX_CLAMP, OnBnClickedTexClamp)
  ON_BN_CLICKED(IDC_TEX_MIRROR_ONCE, OnBnClickedTexMirrorOnce)
  ON_BN_CLICKED(IDC_TEX_BORDER_COLOR, OnBnClickedTexBorderColor)
  ON_BN_CLICKED(IDC_TEX_BORDER_COLOR_BUTTON, OnBnClickedTexBorderColorButton)
  ON_BN_CLICKED(IDC_TEX_FILTER_MAG, OnBnClickedTexFilterMag)
  ON_BN_CLICKED(IDC_TEX_FILTER_MIN, OnBnClickedTexFilgerMin)
  ON_BN_CLICKED(IDC_TEX_FILTER_POINT, OnBnClickedTexFilterPoint)
  ON_BN_CLICKED(IDC_TEX_FILTER_LINEAR, OnBnClickedTexFilterLinear)
  ON_BN_CLICKED(IDC_TEX_FILTER_TRIANGLE, OnBnClickedTexFilterTriangle)
  ON_BN_CLICKED(IDC_TEX_FILTER_GAUSSIAN, OnBnClickedTexFilterGaussian)
  ON_WM_HSCROLL()
  ON_BN_CLICKED(IDC_TEX_FILTER_NONE, OnBnClickedTexFilterNone)
  ON_BN_CLICKED(IDC_TEX_BORDER_ALPHA_BUTTON, OnBnClickedTexBorderAlphaButton)
END_MESSAGE_MAP()



void FileTextureControl::SetFileTextuers(int index)
{
  ResourceTable *r1, *r2;

  m_F1 = m_F2 = NULL;
  if (index < 0) {
    return;
  }

  r1 = m_V1->GetResourceTable();
  r2 = m_V2->GetResourceTable();

  m_F1 = static_cast<FileTexture *> ((*r1)[index]);
  m_F2 = static_cast<FileTexture *> ((*r2)[index]);

  if (m_F1 == NULL) 
    return;

  TextureSetControl();
}

void FileTextureControl::TextureSetFilterType()
{
    if (NULL == m_F1) return;

    int filterType = IDC_TEX_FILTER_NONE;
    switch (m_F1->m_Filter[m_MinMagFilterMode]) {
      case D3DTEXF_NONE:          filterType = IDC_TEX_FILTER_NONE; break;
      case D3DTEXF_POINT:         filterType = IDC_TEX_FILTER_POINT; break;
      case D3DTEXF_LINEAR:        filterType = IDC_TEX_FILTER_LINEAR; break;
      case D3DTEXF_PYRAMIDALQUAD: filterType = IDC_TEX_FILTER_TRIANGLE; break;
      case D3DTEXF_GAUSSIANQUAD:  filterType = IDC_TEX_FILTER_GAUSSIAN; break;
    }
    CheckRadioButton(IDC_TEX_FILTER_NONE, IDC_TEX_FILTER_GAUSSIAN, filterType);
    UpdateData(FALSE);
}

void FileTextureControl::TextureSetAddressMode()
{
    if (NULL == m_F1) return;

    int addressMode = IDC_TEX_WRAP;
    switch (m_F1->m_UVAddr[m_UVAddressMode]) {
      case D3DTADDRESS_WRAP:        addressMode = IDC_TEX_WRAP;   break;
      case D3DTADDRESS_MIRROR:      addressMode = IDC_TEX_MIRROR;   break;
      case D3DTADDRESS_CLAMP:       addressMode = IDC_TEX_CLAMP;      break;
      case D3DTADDRESS_MIRRORONCE:  addressMode = IDC_TEX_MIRROR_ONCE;   break;
      case D3DTADDRESS_BORDER:      addressMode = IDC_TEX_BORDER_COLOR;   break;
    }
    CheckRadioButton(IDC_TEX_WRAP, IDC_TEX_BORDER_COLOR, addressMode);
    UpdateData(FALSE);
}

void FileTextureControl::TextureSetControl()
{
    TextureSetFilterType();
    TextureSetAddressMode();

    m_TexRotate.SetSliderValue( D3DXToDegree(m_F1->m_TexRotate) );
    m_TexTransX.SetSliderValue( m_F1->m_TexTransX );
    m_TexTransY.SetSliderValue( m_F1->m_TexTransY );
    m_TexScaleX.SetSliderValue( m_F1->m_TexScaleX );
    m_TexScaleY.SetSliderValue( m_F1->m_TexScaleY );

    UpdateData(FALSE);
}

// FileTextureControl message handlers

// LightControl message handlers
BOOL FileTextureControl::OnInitDialog()
{
		CDialog::OnInitDialog();

    CheckRadioButton(IDC_TEX_FILTER_NONE, IDC_TEX_FILTER_GAUSSIAN, IDC_TEX_FILTER_POINT);
    CheckRadioButton(IDC_TEX_U, IDC_TEX_V, IDC_TEX_U);
    CheckRadioButton(IDC_TEX_FILTER_MIN, IDC_TEX_FILTER_MAG, IDC_TEX_FILTER_MAG);
    CheckRadioButton(IDC_TEX_WRAP, IDC_TEX_BORDER_COLOR, IDC_TEX_WRAP);

    m_UVAddressMode = eFileTexture_UAddress;
    m_MinMagFilterMode = eFileTexture_MagFilter;

    m_TexRotate.Initialize(-270.0f, 270.0f, 0.0f); 
    m_TexTransX.Initialize(-5.0f, 5.0f, 0.0f);
    m_TexTransY.Initialize(-5.0f, 5.0f, 0.0f);
    m_TexScaleX.Initialize( 0.1f, 10.0f, 1.0f);
    m_TexScaleY.Initialize( 0.1f, 10.0f, 1.0f);

    UpdateData(FALSE);
    return TRUE;
}

void FileTextureControl::OnBnClickedTexU()
{
  m_UVAddressMode = eFileTexture_UAddress;
  TextureSetAddressMode();
}

void FileTextureControl::OnBnClickedTexV()
{
  m_UVAddressMode = eFileTexture_VAddress;
  TextureSetAddressMode();
}

void FileTextureControl::OnBnClickedTexWrap()
{
  if ( (NULL != m_F1) && (NULL != m_F2) ) {
      m_F1->m_UVAddr[m_UVAddressMode] = D3DTADDRESS_WRAP;
      m_F2->m_UVAddr[m_UVAddressMode] = D3DTADDRESS_WRAP;
  }
}

void FileTextureControl::OnBnClickedTexMirror()
{
  if ( (NULL != m_F1) && (NULL != m_F2) ) {
    m_F1->m_UVAddr[m_UVAddressMode] = D3DTADDRESS_MIRROR;
    m_F2->m_UVAddr[m_UVAddressMode] = D3DTADDRESS_MIRROR;
  }
}

void FileTextureControl::OnBnClickedTexClamp()
{
  if ( (NULL != m_F1) && (NULL != m_F2) ) {
    m_F1->m_UVAddr[m_UVAddressMode] = D3DTADDRESS_CLAMP;
    m_F2->m_UVAddr[m_UVAddressMode] = D3DTADDRESS_CLAMP;
  }
}

void FileTextureControl::OnBnClickedTexMirrorOnce()
{
  if ( (NULL != m_F1) && (NULL != m_F2) ) {
    m_F1->m_UVAddr[m_UVAddressMode] = D3DTADDRESS_MIRRORONCE;
    m_F2->m_UVAddr[m_UVAddressMode] = D3DTADDRESS_MIRRORONCE;
  }
}

void FileTextureControl::OnBnClickedTexBorderColor()
{
  if ( (NULL != m_F1) && (NULL != m_F2) ) {
    m_F1->m_UVAddr[m_UVAddressMode] = D3DTADDRESS_BORDER;
    m_F2->m_UVAddr[m_UVAddressMode] = D3DTADDRESS_BORDER;
  }
}

void FileTextureControl::OnBnClickedTexBorderColorButton()
{
  if ( (NULL != m_F1) && (NULL != m_F2) ) {
      CColorDialog dlg;
	    INT_PTR returnValue = dlg.DoModal();
	    if (returnValue == IDOK) {
		      COLORREF color = dlg.GetColor();
		      D3DXCOLOR c;
		      c.r = ((color & 0XFF)) / 255.0f;
		      c.g = ((color & 0x00FF00) >> 8) / 255.0f;
		      c.b = ((color & 0xFF0000) >> 16) / 255.0f;
          c.a = m_F1->m_BorderColor.a;
          m_F1->m_BorderColor = c;
          m_F2->m_BorderColor = c;
    		    	
		      CString msg;
		      msg.Format("Set Border Color to: (%f %f %f)", c.r, c.g, c.b);
		      EchoToStatusArea(msg);
		      UpdateData(FALSE);
	    }
  }
}

void FileTextureControl::OnBnClickedTexBorderAlphaButton()
{
  if ( (NULL != m_F1) && (NULL != m_F2) ) {
      CColorDialog dlg;
	    INT_PTR returnValue = dlg.DoModal();
	    if (returnValue == IDOK) {
		      COLORREF color = dlg.GetColor();
		      D3DXCOLOR c; float a;
		      c.r = ((color & 0XFF)) / 255.0f;
		      c.g = ((color & 0x00FF00) >> 8) / 255.0f;
		      c.b = ((color & 0xFF0000) >> 16) / 255.0f;
          a = c.r + c.g + c.b;
          a = a/3.0f;
          m_F1->m_BorderColor.a = a;
          m_F2->m_BorderColor.a = a;
    		    	
		      CString msg;
		      msg.Format("Set Border Alpha to: (%f)", c.a);
		      EchoToStatusArea(msg);
		      UpdateData(FALSE);
	    }
  }
}

void FileTextureControl::OnBnClickedTexFilterMag()
{
  m_MinMagFilterMode = eFileTexture_MagFilter;
  TextureSetFilterType();
}

void FileTextureControl::OnBnClickedTexFilgerMin()
{
  m_MinMagFilterMode = eFileTexture_MinFilter;
  TextureSetFilterType();
}

void FileTextureControl::OnBnClickedTexFilterNone()
{
  if ( (NULL != m_F1) && (NULL != m_F2) ) {
      m_F1->m_Filter[m_MinMagFilterMode] = D3DTEXF_NONE;
      m_F2->m_Filter[m_MinMagFilterMode] = D3DTEXF_NONE;
  }
}

void FileTextureControl::OnBnClickedTexFilterPoint()
{
  if ( (NULL != m_F1) && (NULL != m_F2) ) {
    m_F1->m_Filter[m_MinMagFilterMode] = D3DTEXF_POINT;
    m_F2->m_Filter[m_MinMagFilterMode] = D3DTEXF_POINT;
  }
}

void FileTextureControl::OnBnClickedTexFilterLinear()
{
  if ( (NULL != m_F1) && (NULL != m_F2) ) {
    m_F1->m_Filter[m_MinMagFilterMode] = D3DTEXF_LINEAR;
    m_F2->m_Filter[m_MinMagFilterMode] = D3DTEXF_LINEAR;
  }
}

void FileTextureControl::OnBnClickedTexFilterTriangle()
{
  if ( (NULL != m_F1) && (NULL != m_F2) ) {
    m_F1->m_Filter[m_MinMagFilterMode] = D3DTEXF_PYRAMIDALQUAD;
    m_F2->m_Filter[m_MinMagFilterMode] = D3DTEXF_PYRAMIDALQUAD;
  }
}

void FileTextureControl::OnBnClickedTexFilterGaussian()
{
  if ( (NULL != m_F1) && (NULL != m_F2) ) {
     m_F1->m_Filter[m_MinMagFilterMode] = D3DTEXF_GAUSSIANQUAD;
     m_F2->m_Filter[m_MinMagFilterMode] = D3DTEXF_GAUSSIANQUAD;
  }
}

void FileTextureControl::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
  if ( (NULL == m_F1) || (NULL == m_F2) )
      return;

  SliderCtrlWithEcho *pSlider;

  if (pScrollBar == (CScrollBar*)&m_TexRotate) {
      pSlider = &m_TexRotate;
      m_F1->m_TexRotate = D3DXToRadian(m_TexRotate.GetSliderValue());
      m_F2->m_TexRotate = D3DXToRadian(m_TexRotate.GetSliderValue());
  } else if (pScrollBar == (CScrollBar*)&m_TexTransX) {
      pSlider = &m_TexTransX;
      m_F1->m_TexTransX = m_TexTransX.GetSliderValue();
      m_F2->m_TexTransX = m_TexTransX.GetSliderValue();
  }else if (pScrollBar == (CScrollBar*)&m_TexTransY) {
      pSlider = &m_TexTransY;
      m_F1->m_TexTransY = m_TexTransY.GetSliderValue();
      m_F2->m_TexTransY = m_TexTransY.GetSliderValue();
  }else if (pScrollBar == (CScrollBar*)&m_TexScaleX) {
      pSlider = &m_TexScaleX;
      m_F1->m_TexScaleX = m_TexScaleX.GetSliderValue();
      m_F2->m_TexScaleX = m_TexScaleX.GetSliderValue();
  }else if (pScrollBar == (CScrollBar*)&m_TexScaleY) {
      pSlider = &m_TexScaleY;
      m_F1->m_TexScaleY = m_TexScaleY.GetSliderValue();
      m_F2->m_TexScaleY = m_TexScaleY.GetSliderValue();
  }else {
    pSlider = NULL;
  }

  CDialog::OnHScroll(nSBCode, nPos, pScrollBar);
  if (NULL != pSlider)
      pSlider->HScroll(nSBCode, nPos);
}