// InteractiveMeshControl.cpp : implementation file
//

#include "stdafx.h"
#include "ClassExampleDlg.h"
#include "InteractiveMeshControl.h"


// InteractiveMeshControl dialog

IMPLEMENT_DYNAMIC(InteractiveMeshControl, CDialog)
InteractiveMeshControl::InteractiveMeshControl(CWnd* pParent /*=NULL*/)
	: CDialog(InteractiveMeshControl::IDD, pParent)
  , m_ShowNormal(FALSE)
  , m_ShowControlGrid(FALSE)
{
}

InteractiveMeshControl::~InteractiveMeshControl()
{
}

void InteractiveMeshControl::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  DDX_Check(pDX, IDC_MESH_NORMAL, m_ShowNormal);
  DDX_Check(pDX, IDC_CONTROL_GRID, m_ShowControlGrid);
  DDX_Control(pDX, IDC_ROW_CV, m_RowSlider);
  DDX_Control(pDX, IDC_COL_CV, m_ColSlider);
  DDX_Control(pDX, IDC_POS_X,  m_PosXSlider);
  DDX_Control(pDX, IDC_POS_Y,  m_PosYSlider);
  DDX_Control(pDX, IDC_POS_Z,  m_PosZSlider);
}


BEGIN_MESSAGE_MAP(InteractiveMeshControl, CDialog)
  ON_BN_CLICKED(IDC_MESH_NORMAL, OnBnClickedMeshNormal)
  ON_BN_CLICKED(IDC_CONTROL_GRID, OnBnClickedControlGrid)
  ON_WM_HSCROLL()
END_MESSAGE_MAP()

void 
InteractiveMeshControl::SetInteractiveMesh(int i)
{
  ResourceTable *r1, *r2;

  r1 = m_MainViewport->GetResourceTable();
  r2 = m_SideViewport->GetResourceTable();

  m_InteractiveMesh1 = static_cast<InteractiveMesh *> ((*r1)[i]);
  m_InteractiveMesh2 = static_cast<InteractiveMesh *> ((*r2)[i]);

  InitializeControl( m_InteractiveMesh1->Resolution() );
}

void 
InteractiveMeshControl::InitializeCVPos()
{
  D3DXVECTOR3 p;
  if (m_InteractiveMesh1 == NULL)
    return;

  m_InteractiveMesh1->SelectedVertexPosition(p);
#define DELTA 0.2f
#define DELTA_Y 2.0f
  m_PosXSlider.Initialize(p.x-DELTA, p.x+DELTA, p.x);
  m_PosYSlider.Initialize(p.y-DELTA_Y, p.y+DELTA_Y, p.y);
  m_PosZSlider.Initialize(p.z-DELTA, p.z+DELTA, p.z);
}

void 
InteractiveMeshControl::InitializeControl(int res)
{
  if (m_InteractiveMesh1 != NULL) {
      m_ShowNormal = m_InteractiveMesh1->DrawNormalIsEnabled();
      m_InteractiveMesh2->SetDrawNormalTo(m_ShowNormal == TRUE);

      m_ShowControlGrid = m_InteractiveMesh2->DrawControlGridIsEnabled();
      m_InteractiveMesh2->SetDrawControlGridTo(m_ShowControlGrid == TRUE);
  }
  
  m_RowSlider.Initialize(0.0f, res-1.0f, 0.0f);
  m_ColSlider.Initialize(0.0f, res-1.0f, 0.0f);
  m_InteractiveMesh1->SelectVertex(0, 0);
  m_InteractiveMesh2->SelectVertex(0, 0);

  InitializeCVPos();

  UpdateData(FALSE);
}


// InteractiveMeshControl message handlers
BOOL InteractiveMeshControl::OnInitDialog()
{
  CDialog::OnInitDialog();

  m_ShowNormal = m_ShowControlGrid = false;
  m_InteractiveMesh1 = m_InteractiveMesh2 = NULL;
  m_RowSlider.Initialize(0.0f, 1.0f, 0.0f);
  m_ColSlider.Initialize(0.0f, 1.0f, 0.0f);
  
  m_PosXSlider.Initialize(-1.0f, 1.0f, 0.0f);
  m_PosYSlider.Initialize(-1.0f, 1.0f, 0.0f);
  m_PosZSlider.Initialize(-1.0f, 1.0f, 0.0f);

  return TRUE;
}

void InteractiveMeshControl::OnBnClickedMeshNormal()
{
  UpdateData(TRUE);
  m_InteractiveMesh1->SetDrawNormalTo( m_ShowNormal == TRUE );
  m_InteractiveMesh2->SetDrawNormalTo( m_ShowNormal == TRUE );
}

void InteractiveMeshControl::OnBnClickedControlGrid()
{
    UpdateData(TRUE);
    m_InteractiveMesh1->SetDrawControlGridTo( m_ShowControlGrid == TRUE);
    m_InteractiveMesh2->SetDrawControlGridTo( m_ShowControlGrid == TRUE);
}


void InteractiveMeshControl::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
  SliderCtrlWithEcho *pSlider;

  int r, c;
  D3DXVECTOR3 p;

  CDialog::OnHScroll(nSBCode, nPos, pScrollBar);

  if ( (pScrollBar == (CScrollBar*) & m_RowSlider) || (pScrollBar == (CScrollBar*) & m_ColSlider) ) {
	      r = int(m_RowSlider.GetSliderValue());
        c = int(m_ColSlider.GetSliderValue());
        
        m_InteractiveMesh1->SelectVertex(r, c);
        m_InteractiveMesh2->SelectVertex(r, c);
        InitializeCVPos();
        
        #define TestAndGetPtr( _sliderBar_)								        \
					if(pScrollBar == (CScrollBar*)&##_sliderBar_##)  {		\
							pSlider = &##_sliderBar_##;												\
					}
        TestAndGetPtr(m_RowSlider)
        TestAndGetPtr(m_ColSlider)
        
  } else {

      m_InteractiveMesh1->SelectedVertexPosition(p);
      if ( (pScrollBar == (CScrollBar*) & m_PosXSlider) ) {
            p.x = m_PosXSlider.GetSliderValue();
            pSlider = &m_PosXSlider;
      } else {
            if (pScrollBar == (CScrollBar*) & m_PosYSlider)  {
                  p.y = m_PosYSlider.GetSliderValue();
                  pSlider = &m_PosYSlider;
            } else {
                  if (pScrollBar == (CScrollBar*) & m_PosZSlider) {
                      p.z = m_PosZSlider.GetSliderValue();
                      pSlider = &m_PosZSlider;
                  }
            }
      }
      m_InteractiveMesh1->SetSelectedVertexPosition(p);
      m_InteractiveMesh2->SetSelectedVertexPosition(p);
  }

  if (pSlider != NULL)
    pSlider->HScroll(nSBCode, nPos);

}