// LightControl.cpp : implementation file
//

#include "stdafx.h"
#include "ClassExampleDlg.h"
#include "LightControl.h"
#include "World.h"
#include ".\lightcontrol.h"


// LightControl dialog

IMPLEMENT_DYNAMIC(LightControl, CDialog)
LightControl::LightControl(CWnd* pParent /*=NULL*/)
	: CDialog(LightControl::IDD, pParent)
	,m_Light(NULL)
  , m_LightIsOn(FALSE)
  , m_ShouldDrawLight(FALSE)
{
}

LightControl::~LightControl()
{
}

void LightControl::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  DDX_Control(pDX, IDC_LGT_COLOR_R, m_RSlider);
  DDX_Control(pDX, IDC_LGT_COLOR_G, m_GSlider);
  DDX_Control(pDX, IDC_LGT_COLOR_B, m_BSlider);
  DDX_Control(pDX, IDC_LGT_COLOR_A, m_ASlider);
  DDX_Control(pDX, IDC_LGT_X,				m_XSlider);	
  DDX_Control(pDX, IDC_LGT_Y,       m_YSlider);
  DDX_Control(pDX, IDC_LGT_Z,				m_ZSlider);
  DDX_Control(pDX, IDC_SPOT_THETA,  m_ThetaSlider);
  DDX_Control(pDX, IDC_SPOT_PHI,		m_PhiSlider);
  DDX_Control(pDX, IDC_SPOT_FALLOFF,m_FallOffSlider);
  DDX_Check(pDX, IDC_LGT_SWITCH, m_LightIsOn);
  DDX_Check(pDX, IDC_LGT_DRAW, m_ShouldDrawLight);
}

BEGIN_MESSAGE_MAP(LightControl, CDialog)
  ON_WM_HSCROLL()
  ON_BN_CLICKED(IDC_LGT_POINT, OnBnClickedLgtPoint)
  ON_BN_CLICKED(IDC_LGT_DIRECTIONAL, OnBnClickedLgtDirectional)
  ON_BN_CLICKED(IDC_LGT_SPOT, OnBnClickedLgtSpot)
  ON_BN_CLICKED(IDC_LGT_POSITION, OnBnClickedLgtPosition)
  ON_BN_CLICKED(IDC_LGT_DIRECTION, OnBnClickedLgtDirection)
  ON_BN_CLICKED(IDC_LGT_DIFFUSE, OnBnClickedLgtDiffuse)
  ON_BN_CLICKED(IDC_LGT_SPECULAR, OnBnClickedLgtSpecular)
  ON_BN_CLICKED(IDC_LGT_AMBIENT, OnBnClickedLgtAmbient)
  ON_BN_CLICKED(IDC_LGT_SWITCH, OnBnClickedLgtSwitch)
  ON_BN_CLICKED(IDC_LGT_DRAW, OnBnClickedLgtDraw)
END_MESSAGE_MAP()

void LightControl::LightSetsControl()
{
	if (m_Light == NULL)
		return;

	D3DXCOLOR c;

	switch (m_LgtColorControls) {
			#define StateCase(_TYPE_)												\
          case eLightColorSliderControls##_TYPE_##:		\
									m_Light->Get##_TYPE_##(c);					\
									break;

					StateCase(Diffuse)
					StateCase(Specular)
					StateCase(Ambient)
			#undef StateCase
	}
	m_RSlider.SetSliderValue(c.r);
	m_GSlider.SetSliderValue(c.g);
	m_BSlider.SetSliderValue(c.b);
	m_ASlider.SetSliderValue(c.a);

	D3DXVECTOR3 p;
	if (m_LgtPosControls == eLightPosSliderControlsPosition)
			m_Light->GetPosition(p);
	else
			m_Light->GetDirection(p);
	m_XSlider.SetSliderValue(p.x);
	m_YSlider.SetSliderValue(p.y);
	m_ZSlider.SetSliderValue(p.z);

  m_ThetaSlider.SetSliderValue( D3DXToDegree(m_Light->GetTheta()) );
	m_PhiSlider.SetSliderValue( D3DXToDegree(m_Light->GetPhi()) );
	m_FallOffSlider.SetSliderValue( m_Light->GetFalloff() );

  m_LightIsOn = m_Light->LightIsSwitchedOn();
  m_ShouldDrawLight = m_Light->ShouldDrawLight();
  UpdateData(FALSE);
}

void LightControl::ControlSetsLight(const D3DXCOLOR &c, const D3DXVECTOR3 &v)
{
	if (m_Light == NULL)
		return;

	switch (m_LgtColorControls) {
			#define StateCase(_TYPE_)												\
          case eLightColorSliderControls##_TYPE_##:		\
									m_Light->Set##_TYPE_##(c);					\
									break;

					StateCase(Diffuse)
					StateCase(Specular)
					StateCase(Ambient)
			#undef StateCase
	}
	
	if (m_LgtPosControls == eLightPosSliderControlsPosition)
			m_Light->SetPosition(v);
  else {
      D3DXVECTOR3 d;
      D3DXVec3Normalize(&d, &v);
			m_Light->SetDirection(d);
  }
  
  OnBnClickedLgtSwitch();
  OnBnClickedLgtDraw();
}

// LightControl message handlers
BOOL LightControl::OnInitDialog()
{
		CDialog::OnInitDialog();

		CheckRadioButton(IDC_LGT_POSITION, 	IDC_LGT_DIRECTIONAL,  IDC_LGT_POSITION);
		CheckRadioButton(IDC_LGT_DIFFUSE, 	IDC_LGT_AMBIENT, 	    IDC_LGT_DIFFUSE);
		CheckRadioButton(IDC_LGT_POINT, 		IDC_LGT_SPOT, 	 	    IDC_LGT_POINT);

		m_LgtColorControls = eLightColorSliderControlsDiffuse;
		m_LgtPosControls = eLightPosSliderControlsPosition;

	 	m_RSlider.Initialize(0.0f, 1.0f, 0.0f);
	 	m_GSlider.Initialize(0.0f, 1.0f, 0.0f);
	 	m_BSlider.Initialize(0.0f, 1.0f, 0.0f);
	 	m_ASlider.Initialize(0.0f, 1.0f, 0.0f);

		m_XSlider.Initialize(-20.0f, 20.0f, 0.0f);
	  m_YSlider.Initialize(-20.0f, 20.0f, 0.0f);
		m_ZSlider.Initialize(-20.0f, 20.0f, 0.0f);

	  m_ThetaSlider.Initialize(0.0f, 120.0f, 0.0f);
		m_PhiSlider.Initialize(0.0f, 120.0f, 0.0f);
		m_FallOffSlider.Initialize(0.0f, 20.0f, 0.0f);

    OnBnClickedLgtPoint();

    return TRUE;
}

void LightControl::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
  	SliderCtrlWithEcho *pSlider;

		if (m_Light == NULL)
			return;
   
      #define TestAndGetPtr( _sliderBar_)								        \
					if(pScrollBar == (CScrollBar*)&##_sliderBar_##)  {		\
							pSlider = &##_sliderBar_##;												\
					}
      TestAndGetPtr(m_RSlider)
      TestAndGetPtr(m_GSlider)
      TestAndGetPtr(m_BSlider)
      TestAndGetPtr(m_ASlider)
      TestAndGetPtr(m_XSlider)
      TestAndGetPtr(m_YSlider)
      TestAndGetPtr(m_ZSlider)
      TestAndGetPtr(m_ThetaSlider)
      TestAndGetPtr(m_PhiSlider)
      TestAndGetPtr(m_FallOffSlider)

		if (pSlider == NULL)
			return;

		D3DXCOLOR 	c;
		D3DXVECTOR3 v;

	 	c.r = m_RSlider.GetSliderValue();
	 	c.g = m_GSlider.GetSliderValue();
	 	c.b = m_BSlider.GetSliderValue();
	 	c.a = m_ASlider.GetSliderValue();

		v.x = m_XSlider.GetSliderValue();
	  v.y = m_YSlider.GetSliderValue();
		v.z = m_ZSlider.GetSliderValue();

		ControlSetsLight(c, v);

	  m_Light->SetTheta(D3DXToRadian(m_ThetaSlider.GetSliderValue()));
		m_Light->SetPhi(D3DXToRadian(m_PhiSlider.GetSliderValue()));
		m_Light->SetFalloff(m_FallOffSlider.GetSliderValue());

   	pSlider->HScroll(nSBCode, nPos);
}


void LightControl::OnBnClickedLgtSpot()
{
  m_Light = theWorld.GetSpotLight();
	LightSetsControl();
}

void LightControl::OnBnClickedLgtPoint()
{
  m_Light = theWorld.GetPointLight();
	LightSetsControl();
}

void LightControl::OnBnClickedLgtDirectional()
{
  m_Light = theWorld.GetDirectionalLight();
	LightSetsControl();
}

void LightControl::OnBnClickedLgtPosition()
{
  UpdateData(TRUE);
	m_LgtPosControls = eLightPosSliderControlsPosition;
  LightSetsControl();
}

void LightControl::OnBnClickedLgtDirection()
{
  UpdateData(TRUE);
	m_LgtPosControls = eLightPosSliderControlsDirection;
  LightSetsControl();
}



#define ImplementColorFunc( _TYPE_ )																\
		void LightControl::OnBnClickedLgt##_TYPE_##() {									\
					UpdateData(TRUE);																					\
					m_LgtColorControls = eLightColorSliderControls##_TYPE_##;	\
          LightSetsControl();                                       \
		}

ImplementColorFunc( Ambient )
ImplementColorFunc( Specular )
ImplementColorFunc( Diffuse )


void LightControl::OnBnClickedLgtSwitch()
{
  UpdateData(TRUE);
  if (m_LightIsOn)
    m_Light->SwitchOnLight();
  else
    m_Light->SwitchOffLight();
}

void LightControl::OnBnClickedLgtDraw()
{
  UpdateData(TRUE);
  if (m_ShouldDrawLight)
      m_Light->EnableDrawLight();
  else
      m_Light->DisableDrawLight();
}
