#include "stdafx.h"
#include "resource.h"
#include "PrimitiveControl.h"
#include ".\primitivecontrol.h"

#ifdef _DEBUG
	#define new DEBUG_NEW
#endif

IMPLEMENT_DYNAMIC(PrimitiveControl, CDialog)
PrimitiveControl::PrimitiveControl(CWnd* pParent /*=NULL*/)
: CDialog(PrimitiveControl::IDD, pParent)
, m_Texture(FALSE)
, m_WireFrame(FALSE)
, m_Normal(FALSE)
, m_Visibility(TRUE)
, m_Colliders(FALSE)
, m_Shading(FALSE)
, m_Blending(FALSE)
, m_Lighting(FALSE)
{
}

PrimitiveControl::~PrimitiveControl()
{
}

void PrimitiveControl::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  DDX_Control(pDX, IDC_TEXTURE_SLIDER, m_TextureIndex);
  DDX_Control(pDX, IDC_COLOR_R, m_ColorR);
  DDX_Control(pDX, IDC_COLOR_G, m_ColorG);
  DDX_Control(pDX, IDC_COLOR_B, m_ColorB);
  DDX_Control(pDX, IDC_COLOR_A, m_ColorA);
  DDX_Check(pDX, IDC_TEXTURE, 		m_Texture);
  DDX_Check(pDX, IDC_WIRE_FRAME, 	m_WireFrame);
  DDX_Check(pDX, IDC_NORMAL, 			m_Normal);
  DDX_Check(pDX, IDC_VISIBILITY, 	m_Visibility);
  DDX_Check(pDX, IDC_COLLIDERS, 	m_Colliders);
  DDX_Check(pDX, IDC_SHADING, 		m_Shading);
  DDX_Check(pDX, IDC_BLENDING, 		m_Blending);
  DDX_Check(pDX, IDC_LIGHTING, 		m_Lighting);
}


BEGIN_MESSAGE_MAP(PrimitiveControl, CDialog)
  ON_WM_HSCROLL()
  ON_WM_CTLCOLOR()
  ON_BN_CLICKED(IDC_TEXTURE,    OnBnClickedTexture)
  ON_BN_CLICKED(IDC_VISIBILITY, OnBnClickedVisibility)
  ON_BN_CLICKED(IDC_SHADING,    OnBnClickedShading)
  ON_BN_CLICKED(IDC_BLENDING,   OnBnClickedBlending)
  ON_BN_CLICKED(IDC_WIRE_FRAME, OnBnClickedWireFrame)
  ON_BN_CLICKED(IDC_COLLIDERS,  OnBnClickedColliders)
  ON_BN_CLICKED(IDC_DRAW_COLOR, OnBnClickedDrawColor)
  ON_BN_CLICKED(IDC_DIFFUSE, OnBnClickedDiffuse)
  ON_BN_CLICKED(IDC_SPECULAR, OnBnClickedSpecular)
  ON_BN_CLICKED(IDC_AMBIENT, OnBnClickedAmbient)
  ON_BN_CLICKED(IDC_EMISSION, OnBnClickedEmissive)
  ON_BN_CLICKED(IDC_SHINGINGNESS, OnBnClickedShingingness)
  ON_BN_CLICKED(IDC_LIGHTING, OnBnClickedLighting)
END_MESSAGE_MAP()

void PrimitiveControl::InitializeRSliderToControlColor()
{
  m_ColorR.Initialize(0.0f, 1.0f, 0.8f);
}

void PrimitiveControl::InitializeRSliderToControlShingingness()
{
  m_ColorR.Initialize(0.1f, 50.0f, 1.0f);
}

void PrimitiveControl::PrimitiveSetsControl()
{
  if (m_Primitive != NULL) {
      m_Texture = m_Primitive->TextureIsEnabled() ? TRUE : FALSE;
      m_Visibility = m_Primitive->VisibilityIsEnabled() ? TRUE : FALSE;
      m_Blending = m_Primitive->BlendingIsEnabled() ? TRUE : FALSE;
      m_Colliders = m_Primitive->ShowCollidersIsEnabled() ? TRUE : FALSE;
      m_Shading = (m_Primitive->GetShadedMode() == D3DSHADE_GOURAUD) ? TRUE : FALSE;
      m_WireFrame = (m_Primitive->GetFillMode() == D3DFILL_WIREFRAME) ? TRUE : FALSE;
      m_Lighting = m_Primitive->LightingIsEnabled() ? TRUE : FALSE;

      m_TextureIndex.SetSliderValue( float(m_Primitive->GetTextureIndex()) );
      PrimitiveSetsColorSliders();
      UpdateData(FALSE);
  }
}

void PrimitiveControl::PrimitiveSetsColorSliders()
{
  if (m_Primitive == NULL) return;

  D3DXCOLOR c(0.0f, 0.0f, 0.0f, 0.0f);

  switch (m_SliderControl)
  {
  	case eSliderControlColor:
    	c = m_Primitive->GetDrawColor();
    	break;

    case eSliderControlShingingness:
      c.r = m_Primitive->GetMaterial().GetPower();
      break;  
    
    #define StateCase(_TYPE_)                                     \
              case eSliderControl##_TYPE_##:                      \
                    m_Primitive->GetMaterial().Get##_TYPE_##(c);  \
                    break;

    StateCase(Emissive)
    StateCase(Ambient)
    StateCase(Diffuse)
    StateCase(Specular)

    #undef StateCase
  }
  m_ColorR.SetSliderValue(c.r);
  m_ColorG.SetSliderValue(c.g);
  m_ColorB.SetSliderValue(c.b);
  m_ColorA.SetSliderValue(c.a);
  UpdateData(FALSE);

  OnBnClickedTexture();   // this makes sure if the texture slider bar is active/inactive
}

void PrimitiveControl::ColorSlidersSetPrimitive()
{
  if (m_Primitive == NULL)
      return;

  D3DXCOLOR c;
  c.r = m_ColorR.GetSliderValue();
  c.g = m_ColorG.GetSliderValue();
  c.b = m_ColorB.GetSliderValue();
  c.a = m_ColorA.GetSliderValue();

  switch (m_SliderControl) {
    case eSliderControlColor:           m_Primitive->SetDrawColor(c); break;
    case eSliderControlShingingness:    m_Primitive->GetMaterial().SetPower( c.r ); break;
          break;

      #define StateCase(_TYPE_)                                  \
              case eSliderControl##_TYPE_##:                     \
                    m_Primitive->GetMaterial().Set##_TYPE_##(c); \
                    break;

    StateCase(Emissive)
    StateCase(Ambient)
    StateCase(Diffuse)
    StateCase(Specular)
    #undef StateCase
    
  }
}


// PrimitiveControl message handlers

BOOL PrimitiveControl::OnInitDialog()
{
   CDialog::OnInitDialog();

	m_TextureIndex.Initialize(-1.0f, 15.0f, 0.0f);

  m_ColorR.Initialize(0.0f, 1.0f, 0.8f);
  m_ColorG.Initialize(0.0f, 1.0f, 0.8f);
  m_ColorB.Initialize(0.0f, 1.0f, 0.8f);
  m_ColorA.Initialize(0.0f, 1.0f, 1.0f);

  m_Primitive = NULL;
  m_SliderControl = eSliderControlColor;

  CheckRadioButton(IDC_DRAW_COLOR, IDC_SPECULAR, IDC_DRAW_COLOR);

  return TRUE;
}

// I first saw the follow code in Vinh Weddle [CSS451 Winter 2003]'s mp0 solution
// Here is what Vinh said:
//		the following code is based on code that i found on msdn regarding
//		the OnCtlColor() member function
//
HBRUSH PrimitiveControl::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{
	HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);
	
	// TODO: Change any attributes of the DC here
	if (pWnd->GetDlgCtrlID() == IDC_R_LABEL)
	{
		pDC->SetTextColor( RGB(255,0,0) );
		pDC->SetBkMode(TRANSPARENT);
	}
	else if (pWnd->GetDlgCtrlID() == IDC_G_LABEL)
	{
		pDC->SetTextColor( RGB(0,255,0) );
		pDC->SetBkMode(TRANSPARENT);
	}
	else if (pWnd->GetDlgCtrlID() == IDC_B_LABEL)
	{
		pDC->SetTextColor( RGB(0,0,255) );
		pDC->SetBkMode(TRANSPARENT);
  }

  return hbr;
}

void PrimitiveControl::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
  // For Update the SliderCtrlWithEcho's echoing
  SliderCtrlWithEcho *pSlider = NULL;

			#define TestAndGetPtr( _sliderBar_)								        \
					if(pScrollBar == (CScrollBar*)&##_sliderBar_##)  {		\
							pSlider = &##_sliderBar_##;												\
					} 																										\

	TestAndGetPtr(m_ColorR)
	TestAndGetPtr(m_ColorG)
  TestAndGetPtr(m_ColorB)
  TestAndGetPtr(m_ColorA)

  //
  // Make sure the echoing of SliderCtrlWithEcho shows up
  if (pSlider != NULL) {
        ColorSlidersSetPrimitive();
  } else {
    // Texture slider moved ...
    pSlider = & m_TextureIndex;
    if (m_Primitive != NULL) {
        m_Primitive->SetTextureIndex( int(m_TextureIndex.GetSliderValue()) );
        m_FileTextureControl->SetFileTextuers( m_Primitive->GetTextureIndex() );
    }
  }
   pSlider->HScroll(nSBCode, nPos);
}

void PrimitiveControl::OnBnClickedTexture()
{
  UpdateData(TRUE);
  if (m_Primitive != NULL) {
      if (m_Texture) {
          m_Primitive->EnableTexture();
          m_Primitive->SetTextureIndex( int(m_TextureIndex.GetSliderValue()) );
          m_TextureIndex.EnableWindow(TRUE);
          m_FileTextureControl->ShowWindow(SW_SHOW);
      } else {
          m_Primitive->DisableTexture();
          m_TextureIndex.EnableWindow(FALSE);
          m_FileTextureControl->ShowWindow(SW_HIDE);
      }
  }
}

void PrimitiveControl::OnBnClickedVisibility()
{
  UpdateData(TRUE);
  if (m_Primitive != NULL) {
      if (m_Visibility) {
          m_Primitive->EnableVisibility();
      } else {
          m_Primitive->DisableVisibility();
      }
  }
}

void PrimitiveControl::OnBnClickedShading()
{
  UpdateData(TRUE);
  if (m_Primitive != NULL) {
      if (m_Shading) {
          m_Primitive->SetShadedMode(D3DSHADE_GOURAUD);
      } else {
          m_Primitive->SetShadedMode(D3DSHADE_FLAT);
      }
  }
}

void PrimitiveControl::OnBnClickedBlending()
{
  UpdateData(TRUE);
  if (m_Primitive != NULL) {
      if (m_Blending) {
          m_Primitive->EnableBlending();
      } else {
          m_Primitive->DisableBlending();
      }
  }
}

void PrimitiveControl::OnBnClickedWireFrame()
{
  UpdateData(TRUE);
  if (m_Primitive != NULL) {
      if (m_WireFrame) {
          m_Primitive->SetFillMode(D3DFILL_WIREFRAME);
      } else {
          m_Primitive->SetFillMode(D3DFILL_SOLID);
      }
  }
}

void PrimitiveControl::OnBnClickedColliders()
{
  UpdateData(TRUE);
  if (m_Primitive != NULL) {
      if (m_Colliders) {
          m_Primitive->EnableShowColliders();
      } else {
          m_Primitive->DisableShowColliders();
      }
  }
}

void PrimitiveControl::OnBnClickedDrawColor()
{
  m_SliderControl = eSliderControlColor;
  InitializeRSliderToControlColor();
  PrimitiveSetsColorSliders();
}


#define ImplementMaterialFunctions( _TYPE_, _VALUE_ )   \
    void PrimitiveControl::OnBnClicked##_TYPE_##()      \
    {                                                   \
        m_SliderControl = eSliderControl##_TYPE_##;     \
        InitializeRSliderToControl##_VALUE_##();        \
        PrimitiveSetsColorSliders();                    \
    }

ImplementMaterialFunctions(Diffuse, Color)
ImplementMaterialFunctions(Emissive, Color)
ImplementMaterialFunctions(Ambient, Color)
ImplementMaterialFunctions(Specular, Color)
ImplementMaterialFunctions(Shingingness, Shingingness)


void PrimitiveControl::OnBnClickedLighting()
{
  UpdateData(TRUE);
  if (m_Primitive != NULL) {
      if (m_Lighting) {
          m_Primitive->EnableLighting();
      } else {
          m_Primitive->DisableLighting();
      }
  }
}
