#include "stdafx.h"
#include "SceneNodeControl.h"
#include "World.h"

#ifdef _DEBUG
  #define new DEBUG_NEW
#endif

IMPLEMENT_DYNAMIC(SceneNodeControl, CDialog)

SceneNodeControl::SceneNodeControl(CWnd* pParent)
: CDialog(SceneNodeControl::IDD, pParent)
, m_ShowPivot(FALSE)
, m_SceneNode(NULL)
, m_ShowAxisFrame(FALSE)
{
}

SceneNodeControl::~SceneNodeControl()
{
}

void SceneNodeControl::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  DDX_Control(pDX, IDC_X_SLIDER, m_XSlider);
  DDX_Control(pDX, IDC_Y_SLIDER, m_YSlider);
  DDX_Control(pDX, IDC_Z_SLIDER, m_ZSlider);
  DDX_Check(pDX, IDC_SHOW_PIVOT, m_ShowPivot);
  DDX_Check(pDX, IDC_SHOW_AXIS_FRAME, m_ShowAxisFrame);
}


BEGIN_MESSAGE_MAP(SceneNodeControl, CDialog)
  ON_WM_HSCROLL()  
  ON_WM_CTLCOLOR()
  ON_BN_CLICKED(IDC_TRANSLATE, OnBnClickedTranslate)
  ON_BN_CLICKED(IDC_ROTATE, OnBnClickedRotate)
  ON_BN_CLICKED(IDC_SCALE, OnBnClickedScale)
  ON_BN_CLICKED(IDC_PIVOT, OnBnClickedPivot)
  ON_BN_CLICKED(IDC_SHOW_PIVOT, OnBnClickedShowPivot)
  ON_BN_CLICKED(IDC_SHOW_AXIS_FRAME, OnBnClickedShowAxisFrame)
END_MESSAGE_MAP()

void SceneNodeControl::ResetSceneNode()
{
  m_ShowPivot = FALSE;
  m_SliderControl = eSliderControlTranslate;
  InitializeSlidersToTranslate();
  UpdateData(FALSE);
  ControlSetsSceneNode();

  if (m_SceneNode != NULL)
  {
    m_SceneNode->m_XForm.ResetTransformation();
    m_SceneNode->m_XForm.SetPivotTo(0.0f, 0.0f, 0.0f);
  }
}

void SceneNodeControl::InitializeSlidersToPivot()
{
  InitializeSlidersToTranslate();
}

void SceneNodeControl::InitializeSlidersToTranslate()
{
    m_XSlider.Initialize(-20.0f, 20.0f, 0.0f);
    m_YSlider.Initialize(-20.0f, 20.0f, 0.0f);
    m_ZSlider.Initialize(-20.0f, 20.0f, 0.0f);
}

void SceneNodeControl::InitializeSlidersToScale()
{
    m_XSlider.Initialize(0.1f, 10.0f, 1.0f);
    m_YSlider.Initialize(0.1f, 10.0f, 1.0f);
    m_ZSlider.Initialize(0.1f, 10.0f, 1.0f);
}

void SceneNodeControl::InitializeSlidersToRotate()
{
    m_XSlider.Initialize(-270.0f, 270.0f, 0.0f);
    m_YSlider.Initialize(-270.0f, 270.0f, 0.0f);
    m_ZSlider.Initialize(-270.0f, 270.0f, 0.0f);
    m_CurrentRotation.x = 0.0f;
    m_CurrentRotation.y = 0.0f;
    m_CurrentRotation.z = 0.0f;
}

void SceneNodeControl::SceneNodeSetsControl()
{
  if (m_SceneNode != NULL)
  {
    D3DXVECTOR3 v;
    switch (m_SliderControl)
    {
    case eSliderControlTranslate:   v = m_SceneNode->m_XForm.Translation(); break;
    case eSliderControlScale:       v = m_SceneNode->m_XForm.Scale(); break;
    case eSliderControlPivot:       v = m_SceneNode->m_XForm.XFormPivot(); break;
    case eSliderControlRotate:      
      v.x = 0.0f;
      v.y = 0.0f;
      v.z = 0.0f;
      m_CurrentRotation = v;
      break;
    }
    m_XSlider.SetSliderValue(v.x);
    m_YSlider.SetSliderValue(v.y);
    m_ZSlider.SetSliderValue(v.z);

    m_ShowPivot = m_SceneNode->ShowPivotIsEnabled();
    m_ShowAxisFrame = m_SceneNode->ShowAxisFrameIsEnabled();
    UpdateData(FALSE);
  }
}

void SceneNodeControl::ControlSetsSceneNode()
{
  if (m_SceneNode != NULL)
  {
    D3DXVECTOR3 v(m_XSlider.GetSliderValue(), m_YSlider.GetSliderValue(), m_ZSlider.GetSliderValue());
    switch (m_SliderControl)
    {
    case eSliderControlTranslate:   m_SceneNode->m_XForm.SetTranslationTo(v); break;
    case eSliderControlRotate:      
      {
            D3DXVECTOR3 deltaV = v - m_CurrentRotation;
            m_SceneNode->m_XForm.UpdateRotationXByDegree(deltaV.x); 
            m_SceneNode->m_XForm.UpdateRotationYByDegree(deltaV.y); 
            m_SceneNode->m_XForm.UpdateRotationZByDegree(deltaV.z); 
            m_CurrentRotation = v;
      }
      break;
    case eSliderControlScale:       m_SceneNode->m_XForm.SetScaleTo(v); break;
    case eSliderControlPivot:       m_SceneNode->m_XForm.SetPivotTo(v); break;
    }

    m_SceneNode->SetShowPivot(m_ShowPivot ? true : false );
  }
}

// SceneNodeControl message handlers

BOOL SceneNodeControl::OnInitDialog()
{
  CDialog::OnInitDialog();

  InitializeSlidersToTranslate();
  CheckRadioButton(IDC_TRANSLATE, IDC_PIVOT, IDC_TRANSLATE);
  m_ShowPivot = FALSE;
  m_SliderControl = eSliderControlTranslate;

  UpdateData(FALSE);
  
  return TRUE;
}

// I first saw the follow code in Vinh Weddle [CSS451 Winter 2003]'s mp0 solution
// Here is what Vinh said:
//		the following code is based on code that i found on msdn regarding
//		the OnCtlColor() member function
//
HBRUSH SceneNodeControl::OnCtlColor(CDC* pDC, CWnd* pWnd, UINT nCtlColor) 
{ 
  HBRUSH hbr = CDialog::OnCtlColor(pDC, pWnd, nCtlColor);

  // TODO: Change any attributes of the DC here
  if (pWnd->GetDlgCtrlID() == IDC_X_LABEL)
  {
    pDC->SetTextColor( RGB(255,0,0) );
    pDC->SetBkMode(TRANSPARENT);
  }
  else if (pWnd->GetDlgCtrlID() == IDC_Y_LABEL)
  {
    pDC->SetTextColor( RGB(0,255,0) );
    pDC->SetBkMode(TRANSPARENT);
  }
  else if (pWnd->GetDlgCtrlID() == IDC_Z_LABEL)
  {
    pDC->SetTextColor( RGB(0,0,255) );
    pDC->SetBkMode(TRANSPARENT);
  }
  return hbr;
}

void SceneNodeControl::OnBnClickedTranslate()
{
  m_SliderControl = eSliderControlTranslate;
  InitializeSlidersToTranslate();
  SceneNodeSetsControl();
}

void SceneNodeControl::OnBnClickedRotate()
{
  m_SliderControl = eSliderControlRotate;
  InitializeSlidersToRotate();
  SceneNodeSetsControl();
}

void SceneNodeControl::OnBnClickedScale()
{
  m_SliderControl = eSliderControlScale;
  InitializeSlidersToScale();
  SceneNodeSetsControl();
}

void SceneNodeControl::OnBnClickedPivot()
{
  m_SliderControl = eSliderControlPivot;
  InitializeSlidersToTranslate();
  SceneNodeSetsControl();
}

void SceneNodeControl::OnBnClickedShowPivot()
{
  UpdateData(TRUE);
  if (m_SceneNode != NULL)
    m_SceneNode->SetShowPivot(m_ShowPivot ? true : false);
}

void SceneNodeControl::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar)
{
  // For Update the SliderCtrlWithEcho's echoing
  SliderCtrlWithEcho *pSlider = NULL;

#define TestAndGetPtr( _sliderBar_) \
  if(pScrollBar == (CScrollBar*)&##_sliderBar_##)  {		\
  pSlider = &##_sliderBar_##;												\
  } 																										\

  TestAndGetPtr(m_XSlider)
  TestAndGetPtr(m_YSlider)
  TestAndGetPtr(m_ZSlider)

  //
  // Make sure the echoing of SliderCtrlWithEcho shows up
  if (pSlider != NULL)
      pSlider->HScroll(nSBCode, nPos);

  ControlSetsSceneNode();
}

void SceneNodeControl::OnBnClickedShowAxisFrame()
{
  UpdateData(TRUE);
  if (m_SceneNode != NULL)
    m_SceneNode->SetShowAxisFrame(m_ShowAxisFrame ? true : false);
}

void SceneNodeControl::SetSceneNode(SceneNode *s)
{
  m_SceneNode = s;
  if(m_SceneNode)
  {
    CString msg;
    msg.Format("Active SceneNode set to %s",m_SceneNode->GetName());
    EchoToStatusArea(msg);
  } 
  SceneNodeSetsControl();
}