#include "stdafx.h"
#include "SceneTreeDialog.h"
#include "World.h"
#include ".\scenetreedialog.h"


// SceneTreeDialog dialog

IMPLEMENT_DYNAMIC(SceneTreeDialog, CDialog)
SceneTreeDialog::SceneTreeDialog(CWnd* pParent /*=NULL*/)
: CDialog(SceneTreeDialog::IDD, pParent),
m_InteractiveMeshControl(NULL)
{
}

SceneTreeDialog::~SceneTreeDialog()
{
  for(unsigned int i=0; i<m_DataNodes.size(); i++)
  {
    TreeNodeData* pNode = m_DataNodes[i];
    delete pNode;
  }
}

void SceneTreeDialog::DoDataExchange(CDataExchange* pDX)
{
  CDialog::DoDataExchange(pDX);
  DDX_Control(pDX, IDC_SCENE_TREE, m_SceneTree);
}


BEGIN_MESSAGE_MAP(SceneTreeDialog, CDialog)
  ON_NOTIFY(TVN_SELCHANGED, IDC_SCENE_TREE, OnTvnSelchangedSceneTree)
END_MESSAGE_MAP()

void SceneTreeDialog::BuildSceneTree(SceneNode* pNode, HTREEITEM hParentNode)
{
  if(NULL == pNode) return;
  std::string name(pNode->GetName());
  name += " node";
  HTREEITEM hItem = m_SceneTree.InsertItem(name.c_str(),hParentNode);
  TreeNodeData* data = new TreeNodeData();
  data->m_pSceneNode = pNode;
  m_DataNodes.push_back(data);
  m_SceneTree.SetItemData(hItem, (DWORD_PTR)data);

  for(unsigned int i=0; i<pNode->NumChildren(); i++)
  {
    SceneNode* pChildNode = pNode->GetChildNode(i);
    BuildSceneTree(pChildNode,hItem);
  }
}
BOOL SceneTreeDialog::OnInitDialog()
{
  CDialog::OnInitDialog();

  LinkControlToPlaceHolder(this,IDC_SCENE_NODE_CTRL_PLACEHOLDER,IDD_SCENE_NODE_CTRL,&m_SceneNodeControl);
  LinkControlToPlaceHolder(this,IDC_PRIMITIVE_CONTROL_PLACEHOLDER,IDD_PRIMITIVE_CONTROL,&m_PrimitiveControl);

  BuildSceneTree(theWorld.GetRootNode());
  m_SceneTree.SelectItem(m_SceneTree.GetFirstVisibleItem());

  return TRUE;  // return TRUE unless you set the focus to a control
}

void SceneTreeDialog::OnTvnSelchangedSceneTree(NMHDR *pNMHDR, LRESULT *pResult)
{
  LPNMTREEVIEW pNMTreeView = reinterpret_cast<LPNMTREEVIEW>(pNMHDR);

  HTREEITEM hItem = pNMTreeView->itemNew.hItem;
  TreeNodeData* pData = reinterpret_cast<TreeNodeData*>(m_SceneTree.GetItemData(hItem));

  if(pData->m_pSceneNode)
  {
    m_SceneNodeControl.ShowWindow(SW_SHOW);
    m_PrimitiveControl.ShowWindow(SW_HIDE);
    if (m_InteractiveMeshControl != NULL)
      m_InteractiveMeshControl->ShowWindow(SW_HIDE);
    m_SceneNodeControl.SetSceneNode(pData->m_pSceneNode);
    DrawablePrimitive* pDrawable = dynamic_cast<UWBGL_D3D::DrawablePrimitive*>(pData->m_pSceneNode->GetPrimitive());
    if(pDrawable)  {
        m_PrimitiveControl.ShowWindow(SW_SHOW);
        m_PrimitiveControl.SetPrimitive(pDrawable);

        if (pDrawable->TextureIsEnabled()) {
          m_FileTextureControl->ShowWindow(SW_SHOW);
          m_FileTextureControl->SetFileTextuers( pDrawable->GetTextureIndex() );
        }

        InteractiveMeshPrimitive *p = dynamic_cast<UWBGL_D3D::InteractiveMeshPrimitive*>(pDrawable);
        if (p != NULL) {
          m_InteractiveMeshControl->ShowWindow(SW_SHOW);
          m_InteractiveMeshControl->SetInteractiveMesh( p->GetMeshIndex() );
        }

    }
  } else {
    m_SceneNodeControl.ShowWindow(SW_HIDE);
    m_PrimitiveControl.ShowWindow(SW_HIDE);
    m_FileTextureControl->ShowWindow(SW_HIDE);
    m_InteractiveMeshControl->ShowWindow(SW_HIDE);
  }

  *pResult = 0;
}