#include "uwbgl_D3DCamera.h"


using namespace UWBGL_D3D;

Camera::Camera()
: m_EyePosition(1,1,1)
, m_AtPosition(0,0,0)
, m_UpVector(0,1,0)
, m_Near(1.0f)
, m_Far(100.f)
, m_bCameraVisible(false)
{
  m_FieldOfView = D3DX_PI/4.0f;
}

Camera::~Camera()
{
}

void Camera::Set(const D3DXVECTOR3& eye, const D3DXVECTOR3& at, const D3DXVECTOR3& up, float fov, float near_clip, float far_clip)
{
  SetView(eye, at, up);
  m_FieldOfView = fov;
  m_Near = near_clip;
  m_Far = far_clip;
}

void Camera::SetView(const D3DXVECTOR3& eye, const D3DXVECTOR3& at, const D3DXVECTOR3& up)
{
  m_EyePosition = eye;
  m_AtPosition = at;
  m_UpVector = up;
}
float Camera::GetViewDistance() const
{
  //view distance is the distance between the eye and at positions
  D3DXVECTOR3 distVector = m_AtPosition - m_EyePosition;
  float length = D3DXVec3Length(&distVector);
  return length;  
}

void Camera::DefineViewAndProjection(float aspectRatio, LPDIRECT3DDEVICE9 pDevice) const
{
  if(NULL == pDevice) return;
  // Set up our view matrix. A view matrix can be defined given an eye point,
  // a point to lookat, and a direction for which way is up. Here, we set the
  // eye five units back along the z-axis and up three units, look at the 
  // origin, and define "up" to be in the y-direction.
  D3DXMATRIX matView;
  D3DXMatrixLookAtRH( &matView, &m_EyePosition, &m_AtPosition, &m_UpVector);
  pDevice->SetTransform( D3DTS_VIEW, &matView );

  // For the projection matrix, we set up a perspective transform (which
  // transforms geometry from 3D view space to 2D viewport space, with
  // a perspective divide making objects smaller in the distance). To build
  // a perpsective transform, we need the field of view (1/4 pi is common),
  // the aspect ratio, and the near and far clipping planes (which define at
  // what distances geometry should be no longer be rendered).
  D3DXMATRIX matProj;
  D3DXMatrixPerspectiveFovRH( &matProj, m_FieldOfView, aspectRatio, m_Near, m_Far);
  pDevice->SetTransform( D3DTS_PROJECTION, &matProj );
}