#include "uwbgl_D3DCamera.h"
#include "uwbgl_D3DSceneNode.h"
#include "uwbgl_D3DCirclePrimitive.h"
#include "uwbgl_D3DMeshPrimitive.h"
#include "uwbgl_D3DLinePrimitive.h"


using namespace UWBGL_D3D;


void Camera::Draw()
{
  SceneNode eyeNode, atNode;
  D3DXVECTOR3 d = m_EyePosition - m_AtPosition;
  LinePrimitive *sight = new LinePrimitive();
  sight->SetEnds(0.0f, 0.0f, 0.0f, d.x, d.y, d.z);
  sight->SetDrawColor(D3DXCOLOR(0.0f, 0.0f, 0.0f, 0.0f));
  sight->SetShadedMode(D3DSHADE_FLAT);

  eyeNode.m_XForm.SetTranslationTo(m_EyePosition);
  eyeNode.m_XForm.SetScaleTo(3.0f);
  eyeNode.EnableShowPivot();

  atNode.m_XForm.SetTranslationTo(m_AtPosition);
  atNode.EnableShowPivot();
  atNode.EnableShowAxisFrame();
  atNode.SetPrimitive( sight );


  atNode.DrawNode();
  eyeNode.DrawNode();
}

void Camera::DrawRailRepresentation(LPDIRECT3DDEVICE9 device)
{
  float radius = GetViewDistance();
  const int num_slices = 6; // on each of the hemi-sphere

  // the horizontal slices
  float delta = M_PI/(2*num_slices);

  SceneNode node;
  CirclePrimitive* pCircle = new CirclePrimitive(radius);
  node.SetPrimitive(pCircle);

  pCircle->SetFillMode(D3DFILL_WIREFRAME);
  pCircle->SetDrawColor(D3DXCOLOR(0.7f, 0.7f, 0.7f, 0.0f));
  node.m_XForm.UpdateRotationXByDegree(90.0f);
  node.m_XForm.SetTranslationTo(m_AtPosition);
  node.DrawNode();

  for(int i=1; i<=num_slices; i++)
  {
    float theta = i*delta;
    float y = radius * sin(theta);
    float r = radius * cos(theta);
    node.m_XForm.SetTranslationTo(m_AtPosition.x,m_AtPosition.y+y,m_AtPosition.z);
    pCircle->SetRadius(r);
    node.DrawNode();

    node.m_XForm.SetTranslationTo(m_AtPosition.x,m_AtPosition.y-y,m_AtPosition.z);
    node.DrawNode();
  }
}