#include "uwbgl_D3DCamera.h"

using namespace UWBGL_D3D;

static D3DXVECTOR3 sYAxis(0.0f, 1.0f, 0.0f);


void Camera::ComputeCameraFrame(float &d, D3DXVECTOR3 &v, D3DXVECTOR3 &u, D3DXVECTOR3 &w)
{
    v = m_AtPosition - m_EyePosition;
    d = D3DXVec3Length(&v);
    v = v * (1.0f/d);

    D3DXVec3Cross(&w, &m_UpVector, &v);
    D3DXVec3Normalize(&w, &w);

    D3DXVec3Cross(&u, &v, &w);
    D3DXVec3Normalize(&u, &u);
}


void Camera::RotateCameraY(float dx)
{
  float dist;
  D3DXVECTOR3 u, v, w;
  ComputeCameraFrame(dist, v, u, w);
	// 1. rotate in the direction of fW by certain -dx degree ...
	//     in the fW direction, this is rotation wrt to fU vector
	// 
	// we DO NOT want to rotate wrt to u, but then, what are we rotating wrt? (the UP vector!)
	if ( v.y != 0.0 ) {
		D3DXMATRIX m;

		float sign = 1.0f;
		
		if (D3DXVec3Dot(&sYAxis, &u) < 0) {
			sign = -1.0f;
		}

		D3DXVECTOR3 axis = sYAxis * sign;

		D3DXMatrixRotationAxis(&m, &axis, -dx);

		D3DXVec3TransformCoord(&v, &v, &m);

		D3DXVECTOR3 tv(v.x, 0, v.z);

		D3DXVec3Cross(&w, &axis, &tv);
		D3DXVec3Normalize(&w, &w);
		//
		// Get the new new U vector 
		D3DXVec3Cross(&u, &v, &w);
		D3DXVec3Normalize(&u, &u);
		m_UpVector = u;
		m_EyePosition = m_AtPosition - (dist * v);
	}	
}

	
void  Camera::RotateCameraX(
	float dy
)
{
    float dist;
    D3DXVECTOR3 u, v, w;
    ComputeCameraFrame(dist, v, u, w);

		// 2. now rotate in the direction of fU by certain dy degree
		//			fU direction rotation is wrt to fW
		D3DXMATRIX m;
		D3DXMatrixRotationAxis(&m, &w, dy);
		D3DXVec3TransformCoord(&v, &v, &m);
		D3DXVec3TransformCoord(&u, &u, &m);
		//
		// Get the new new W vector 
		D3DXVec3Cross(&w, &u, &v);
		D3DXVec3Normalize(&w, &w);
		m_UpVector = u;
    m_EyePosition = m_AtPosition - (dist * v);
}