#include "uwbgl_D3DGraphicsSystem.h"
#include "uwbgl_D3DDeviceVertexFormat.h"
#include "uwbgl_D3DCirclePrimitive.h"
#include "uwbgl_D3DCollider.h"

using namespace UWBGL_D3D;

CirclePrimitive::CirclePrimitive() 
{
  SetCircle(D3DXVECTOR3(0.0f, 0.0f, 0.0f), 0.0f);
}

CirclePrimitive::CirclePrimitive(const D3DXVECTOR3 &c, float r)
{
  SetCircle(c, r);
}

CirclePrimitive::CirclePrimitive(float radius)
{
  SetCircle(D3DXVECTOR3(0.0f, 0.0f, 0.0f), radius);
}

CirclePrimitive::~CirclePrimitive() 
{}

float CirclePrimitive::GetRadius() 
{
  D3DXVECTOR3 d;
  float r = 0.0f;
  if (NumVertices() > 1)
  {
    D3DXVECTOR3 d1, d2, d3;
    d1 = m_Vertices[0];
    d2 = m_Vertices[1];
    d3 = d1 - d2;
    
    d = m_Vertices[0] - m_Vertices[1];
    r = D3DXVec3Length(&d);
  }
  return r;
}
void CirclePrimitive::GetColliderList(ColliderList &list)
{
  if (NumVertices() < 2)
     return;

  Collider c( *(GetCenter()), GetRadius() );
  list.Append(c);
}
const D3DXVECTOR3* CirclePrimitive::GetCenter() const
{
  if (NumVertices() > 1)
    return &(m_Vertices[0]);
  else 
    return NULL;
}
void CirclePrimitive::SetFirstVertex(const D3DXVECTOR3 &p)
{
  DrawablePrimitive::SetFirstVertex(p);
  if (NumVertices() > 1)
    ReplaceVertexAt(1, p);
}

void CirclePrimitive::AppendVertex(const D3DXVECTOR3 &p)
{
  if (NumVertices() < 2)
    DrawablePrimitive::AppendVertex(p);
  else
    ReplaceVertexAt(1, p);
}


void CirclePrimitive::SetCircle(const D3DXVECTOR3 &p, float r)
{
  D3DXVECTOR3 rp(p.x+r, p.y, 0.0f);
  if (NumVertices() == 0)
    DrawablePrimitive::AppendVertex(p);
  else
    ReplaceVertexAt(0, p);

  AppendVertex(rp);
}

void CirclePrimitive::SetRadius(float r)
{
  if (NumVertices() == 0)
    return;

  D3DXVECTOR3 rp(m_Vertices[0].x+r, m_Vertices[0].y, m_Vertices[0].z);
  AppendVertex(rp);
}

void CirclePrimitive::MovePrimitiveTo(const D3DXVECTOR3 &pt)
{
  if (NumVertices() == 0)
    return;

  D3DXVECTOR3 c = *GetCenter();
  m_Vertices[0] = pt;

  if (NumVertices() == 1)
    return;

  D3DXVECTOR3 d = pt - c;
  m_Vertices[1] += d;
}

void CirclePrimitive::DrawPrimitiveGeometry()
{
  LPDIRECT3DDEVICE9 d3dDevice = GraphicsSystem::GetSystem().GetActiveDevice();
  if (d3dDevice == NULL)
    return;

  float r = GetRadius();
  if (r <= 0.0f)
    return;

  const D3DXVECTOR3 *c = GetCenter();

  // We are going to draw this circle with TriangleFan
  const int kNumPts = 100;
  DeviceVertexFormat v[kNumPts+2];
  float delta = (2.0f * M_PI) / kNumPts;
  v[0].m_Point.x = c->x;
  v[0].m_Point.y = c->y;
  v[0].m_Point.z = 0.0f;
  v[0].m_Color = 0xFFFFFFFF;	// This is a white color!!
  v[0].m_U = v[0].m_V = v[0].m_U1 = v[0].m_V1 = 0.5f;
  v[0].m_Normal = D3DXVECTOR3(0,1,0);

  D3DCOLOR co = D3DCOLOR_COLORVALUE(m_DrawColor.r, m_DrawColor.g, m_DrawColor.b, m_DrawColor.a);
  for (int i=0; i<(kNumPts); i++)
  {
    float sinTheta = sinf(i*delta);
    float cosTheta = cosf(i*delta);
    v[i+1].m_Point.x = c->x + (r * sinTheta);
    v[i+1].m_Point.y = c->y + (r * cosTheta);
    v[i+1].m_Point.z = 0.0f;
    v[i+1].m_Normal = v[0].m_Normal;
    v[i+1].m_Color = co;
    v[i+1].m_U = v[i+1].m_U1 = (sinTheta * 0.5f) + 0.5f;
    v[i+1].m_V = v[i+1].m_V1 = 1.0f - ((cosTheta * 0.5f) + 0.5f);
  }
  v[kNumPts+1] = v[1];








  d3dDevice->DrawPrimitiveUP(D3DPT_TRIANGLEFAN, kNumPts, (CONST void *)v, sizeof(DeviceVertexFormat));

}
