#include "uwbgl_D3DCollider.h"
#include "uwbgl_D3DMeshPrimitive.h"
#include "uwbgl_D3DSceneNode.h"

using namespace UWBGL_D3D;

Collider::Collider(float x, float y, float radius) :
  m_Center(x, y, 0.0f),
  m_Radius(radius)
{
}

Collider::Collider(const D3DXVECTOR3 &c, float radius) :
	m_Center(c),
	m_Radius(radius)
{
  m_Center.z = 0.0f;
}

bool Collider::CollidesWith(const Collider& other, D3DXVECTOR3& delta, float &dist) const
{
  delta = other.m_Center - m_Center;
  dist = D3DXVec3Length(&delta);
  return dist < m_Radius + other.m_Radius;
}

void Collider::DrawCollider()
{
  MeshPrimitive *m = new MeshPrimitive(eSphereMesh);
  m->SetFillMode(D3DFILL_WIREFRAME);
  m->SetShadedMode(D3DSHADE_GOURAUD);
  m->SetDrawColor(D3DXCOLOR(0.9f, 0.5f, 0.5f, 0.0f));
  SceneNode sn;
  sn.SetPrimitive( m );
  sn.m_XForm.SetTranslationTo( m_Center );
  sn.m_XForm.SetScaleTo(m_Radius);
  sn.DrawNode();
}

void Collider::Transform(const D3DXMATRIX& matrix)
{
  // 1. get a point on circufrence
  D3DXVECTOR3 c(m_Center), d;
  c.x += m_Radius;
  
  D3DXVec3TransformCoord(&m_Center, &m_Center, &matrix);
  D3DXVec3TransformCoord(&c, &c, &matrix);
  d = m_Center - c;
  m_Radius = D3DXVec3Length(&d);
}

void ColliderList::DrawColliders()
{
  std::vector<Collider>::iterator it;
  it = begin();
  while (it != end() ) {
    (*it).DrawCollider();
    it++;
  }
}

bool ColliderList::CollidesWith(const ColliderList& other, D3DXVECTOR3& delta, float &dist) const
{
  D3DXVECTOR3 tDelta;
  float tDist;
  bool once = false;

  dist = M_VERY_LARGE;
  for(unsigned int i=0; i<other.size(); i++) {
    bool touch = CollidesWith(other[i], tDelta, tDist);
    if ( (touch) && (tDist < dist) ) {
      dist = tDist;
      delta = tDelta;
      once = true;
    }
  }
  return once;
}


bool ColliderList::CollidesWith(const Collider& other, D3DXVECTOR3& delta, float &dist) const
{
  D3DXVECTOR3 tDelta;
  float tDist;
  bool once = false;

  dist = M_VERY_LARGE;

  std::vector<Collider>::const_iterator it;
  it = begin();
  while (it != end() ) {
    bool touched = (*it).CollidesWith(other, tDelta, tDist);
    if ( (touched) && (tDist < dist) ) {
      dist = tDist;
      delta = tDelta;
      once = true;
    }
    it++;
  }
  return once;
}


void ColliderList::AppendList(const ColliderList& other)
{
  for(unsigned int i=0; i<other.size(); i++) push_back(other[i]);
}

void ColliderList::Append(const Collider& other)
{
  push_back(other);
}

void ColliderList::Transform(const D3DXMATRIX& matrix)
{
  for (unsigned int i=0; i<size(); i++)
    at(i).Transform(matrix);
}
