// UW Bothell Graphics Library Copyright (c) 2004 University of Washington Bothell
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
/////////////////////////////////////////////////////////////////////////////////////////
#pragma once

#include "uwbgl_D3DDefines.h"
#include <vector>

namespace UWBGL_D3D
{
  //Object used for collision detection.
  class Collider
  {
  public:
    Collider(float x, float y, float radius);
    Collider(const D3DXVECTOR3 &c, float radius);

    void DrawCollider();

    //  if CollidesWith() returns ture
    //    delta - vector (not normalized) pointing from this towards other
    //    dist - distance between the two collider
    //  if CollidesWith() return false
    //    delta & dist - undefined
    bool CollidesWith(const Collider& other, D3DXVECTOR3& delta, float& dist) const;
    void Transform(const D3DXMATRIX& matrix);

  private:
    D3DXVECTOR3 m_Center;
    float m_Radius;
  };

  //  Notice ColliderList is a list of actual Colliders. This means, upcon construction, actual
  //  Collider objects will be created. Watch out for memory management, Colliders to be inserted
  //  into the ColliderList should be created on the stack.
  class ColliderList : public std::vector<Collider>
  {
  public:
    ColliderList(){}

    void DrawColliders();

    // List-to-List collision is a n^2 operation. Be careful!! performance can suffer easily!
    bool CollidesWith(const ColliderList& other, D3DXVECTOR3& delta, float& dist) const;
    bool CollidesWith(const Collider& other, D3DXVECTOR3& delta, float& dist) const;
    void Transform(const D3DXMATRIX& matrix);
    void AppendList(const ColliderList& other);
    void Append(const Collider& other);
  };
};