#include "uwbgl_D3DDrawablePrimitiveList.h"
#include "uwbgl_D3DDrawablePrimitive.h"
#include "uwbgl_D3DCollider.h"

using namespace UWBGL_D3D;


DrawablePrimitiveList::DrawablePrimitiveList(bool controlPrimitiveLifetime)
 : m_ControlPrimitiveLifetime(controlPrimitiveLifetime)
{}

DrawablePrimitiveList::~DrawablePrimitiveList() 
{
  EraseList();
}

void DrawablePrimitiveList::EraseList() 
{
  if (m_ControlPrimitiveLifetime) {
		// we must make sure we free the Primitive memory ...
  	std::vector<DrawablePrimitive*>::iterator it;
		it = m_List.begin();
		while ( it != m_List.end() ) {
			DrawablePrimitive *p = (*it);
			delete p;
			it++;
		}
	}

	// clear clears the memory control by <vector> (the links)
  m_List.clear();
}

//////////////////////////////////////////////////////
// List specific methods ...
//
int DrawablePrimitiveList::Count() const { return int(m_List.size()); }
DrawablePrimitive* DrawablePrimitiveList::GetDrawablePrimitiveAt(int i) 
{ 
   return (i<0) ? NULL : ( (i>Count()) ? NULL : m_List[i]) ; 
}

void DrawablePrimitiveList::InsertDrawablePrimitive(DrawablePrimitive *pPrimitive)
{
	m_List.push_back(pPrimitive);
}

void DrawablePrimitiveList::DeleteDrawablePrimitive(DrawablePrimitive *pPrimitive)
{
  // for simplicity, delete from the back ...
  for (int i = (int) (m_List.size()-1); i>=0; i--) {
    DrawablePrimitive *p = m_List[i];
    if (p == pPrimitive) {
      delete p;
      std::vector<DrawablePrimitive*>::iterator it = m_List.begin() + i;
      m_List.erase(it);
    }
  }
}

void DrawablePrimitiveList::DeleteOutOfBoundPrimitives(float minx, float miny, float maxx, float maxy)
{ 
  // for simplicity, delete from the back ...
  for (int i = (int) (m_List.size()-1); i>=0; i--) {
    DrawablePrimitive *p = m_List[i];
    if (p->OutOfBound(minx, miny, maxx, maxy)) {
      delete p;
      std::vector<DrawablePrimitive*>::iterator it = m_List.begin() + i;
      m_List.erase(it);
    }
  }
}


/////////////////////////////////////////////////////
//
// PrimitiveBase methods ...
//
void DrawablePrimitiveList::GetColliderList(ColliderList &list)
{
  std::vector<DrawablePrimitive*>::iterator it;

	it = m_List.begin();
	while ( it != m_List.end() ) {
		ColliderList l;
		(*it)->GetColliderList(l);
		list.AppendList(l);
		++it;
	}
	return;
}

/////////////////////////////////////////////////////
//
// IPrimitive methods ...
//
//
void DrawablePrimitiveList::DrawColliders()
{
  std::vector<DrawablePrimitive*>::iterator it;

	it = m_List.begin();
	while ( it != m_List.end() ) {
		(*it)->DrawColliders();
		++it;
	}
	return;
}


void DrawablePrimitiveList::DrawPrimitive()
{
	if (m_List.empty() )
		return;

	if (!ShouldDrawPrimitive())
		return;

  std::vector<DrawablePrimitive*>::iterator it;
	it = m_List.begin();
	while ( it != m_List.end() ) {
		(*it)->DrawPrimitive();
		++it;
	}

	if (ShowCollidersIsEnabled()) {
		ColliderList list;
		GetColliderList(list);
		list.DrawColliders();
	}
}
