#pragma once

#include <vector>
#include "uwbgl_D3DPrimitiveBase.h"
			// STD vector class

namespace UWBGL_D3D
{
  class DrawablePrimitive;

	class DrawablePrimitiveList : public PrimitiveBase {
		//
		// notice DrawablePrimitiveList is a subclass of PrimitiveBase
		// this says, we must define all the pure virtual functions
		// defined in IPrimitive/PrimitiveBase. More importantly, we are saying
		// DrawablePrimitiveList is also an IPrimitive, meaning we will be 
		// able to "DrawPrimitive()" on this list!
		//
		// Notice, each DrawablePrimitiveList is a list of DrawablePrimitive (not list of IPrimitive),
		// this effectively limits our ability to construct list of lists. This is ok for DrawablePrimitiveList
		// is not meant for building complex graphs of inter-connected geometries. Later on, we will 
		// study the SceneNode class, and that will allow us construct interesting lists of lists.
		//

	private:
		
	protected:
			std::vector<DrawablePrimitive*> m_List;
			bool m_ControlPrimitiveLifetime;
						// be responsible for free-ing the memory associated with the pointer

  public:
		
		DrawablePrimitiveList(bool controlPrimitiveLifetime=true);
		virtual ~DrawablePrimitiveList();

		void 	InsertDrawablePrimitive(DrawablePrimitive *pPrimitive);
					// 
					// inserting a drawable primitive to the end of the list
					//

		int		Count() const;
		DrawablePrimitive* GetDrawablePrimitiveAt(int index);
    void  EraseList();
				
		void DeleteDrawablePrimitive(DrawablePrimitive *pPrimitive);	
					// This is an order-n deletion, will free the memory
					// of primitive. Meanning caller should not refer to primitive-pointer
					// after calling this function
					//

    void DeleteOutOfBoundPrimitives(float minx, float miny, float maxx, float maxy);

		// from PrimitiveBase
		//
		virtual void GetColliderList(ColliderList&);

		// from IPrimitive
		//
		virtual void DrawColliders();
		virtual void DrawPrimitive();
	};
};
