#include "../uwbgl_Defines.h"
#include "uwbgl_D3DFileTexture.h"
#include "uwbgl_D3DGraphicsSystem.h"

using namespace UWBGL_D3D;

FileTexture::~FileTexture()
{
	if (m_Texture != NULL)
		m_Texture->Release();
}

FileTexture::FileTexture(LPDIRECT3DDEVICE9 d, const char* filename) :
  m_BorderColor(0.5f, 0.5f, 0.5f, 1.0f)
{
	m_Texture = NULL;
	m_D3dDevice = d;

  m_TexRotate = 0.0f;
  m_TexTransX = m_TexTransY = 0.0f;
  m_TexScaleX = m_TexScaleY = 1.0f;
  m_UVAddr[0] = m_UVAddr[1] = D3DTADDRESS_WRAP;
  m_Filter[0] = m_Filter[1] = D3DTEXF_POINT;
  
	CreateD3DTexture(d, filename);
}


BOOL FileTexture::CreateD3DTexture(LPDIRECT3DDEVICE9 pd3dDevice, const char* filename)
{
	HRESULT hr;
	if( FAILED( hr = D3DXCreateTextureFromFile(m_D3dDevice, filename, &m_Texture ) ) )
	{
		return FALSE;
	}
	return TRUE;
}

void FileTexture::ActivateTexture(DWORD stage)
{
  static D3DXVECTOR3 zAxis(0.0f, 0.0f, 1.0f);

	if (m_Texture == NULL)
		return;

  	// set up our texture stages for a simple texture copy...
		m_D3dDevice->SetTextureStageState( stage, D3DTSS_COLOROP,   D3DTOP_SELECTARG1 );
				// 
				// Refer to the D3D help, the first parameter is a number (between 0 to something)
				//
				// the second parameter can be ...
				//				D3DTSS_COLOROP
				//				D3DTSS_COLORARG1, ARG2
				//				D3DTSS_ALPHAARG1, ARG2
				//				D3DTSS_MAGFILTER, MINFILTER, MIPFILTER (more and more and tons tons more!!)
				//
				// For Stage 0,
				//
				//		Arg1 is the texture
				//		Arg2 is the color on the object

				// The third parameter can be
				//				D3DTA_TEXTURE
				//				D3DTA_DIFFUSE
				//				D3DTA_MODULATE
				//				D3DTA_SELECTARG1
				//				D3DTA_SELECTARG2 ... (more and more and tons tons more!!!)

		m_D3dDevice->SetTextureStageState( stage, D3DTSS_ALPHAOP,   D3DTOP_SELECTARG1);
				// Now see, are setting the alpha channel in the texture stage, in this
				// case, we say, we _always_ want the alpha information from the file texture
				//

    // ************ NOW ************************
    // 1. Placement, let's compute the transform matrix ...
    ID3DXMatrixStack* pStack =  GraphicsSystem::GetSystem().GetWorldStack();
    pStack->Push();
        
        pStack->LoadIdentity();
        pStack->ScaleLocal( m_TexScaleX, m_TexScaleY, 1.0f );
        pStack->RotateAxis( &zAxis, m_TexRotate);
        D3DXMATRIX *tm = pStack->GetTop();

        // Now since textures are two dimensional entities ... translation is in the third row ...
        tm->_31 = m_TexTransX;
        tm->_32 = m_TexTransY;
        
        // Enable Texture coordinate transform
			  m_D3dDevice->SetTextureStageState(0 /* Texture Stage */, D3DTSS_TEXTURETRANSFORMFLAGS, 
											D3DTTFF_COUNT2 /* Says the first three element */);
        // Load the matrix from Stack into Texture transform ...
        m_D3dDevice->SetTransform(D3DTS_TEXTURE0, pStack->GetTop() );
        // now, we are done with using the satck
    pStack->Pop();

    // 2. Set up Texture filtering ...
    m_D3dDevice->SetSamplerState(0 /* Texture Stage */, D3DSAMP_MAGFILTER, m_Filter[eFileTexture_MagFilter]);
    m_D3dDevice->SetSamplerState(0 /* Texture Stage */, D3DSAMP_MINFILTER, m_Filter[eFileTexture_MinFilter]);
    m_D3dDevice->SetSamplerState(0 /* Texture Stage */, D3DSAMP_MIPFILTER, m_Filter[eFileTexture_MinFilter]);
                                        // Possible filter values: NONE, POINT, LINEAR, PYRAMIDALQUAD, GAUSSIANQUAD

    // 3. Set up Texture Addressing ...
    m_D3dDevice->SetSamplerState(0 /* Texture Stage */, D3DSAMP_ADDRESSU, m_UVAddr[eFileTexture_UAddress]);
    m_D3dDevice->SetSamplerState(0 /* Texture Stage */, D3DSAMP_ADDRESSV, m_UVAddr[eFileTexture_VAddress]);
                                       // Possible addressing values: WRAP, MIRROR, CLAMP, BORDER, MIRRORONCE
    D3DCOLOR nc = D3DCOLOR_COLORVALUE(m_BorderColor.r, m_BorderColor.g, m_BorderColor.b, m_BorderColor.a);
    m_D3dDevice->SetSamplerState(0 /* Texture Stage */, D3DSAMP_BORDERCOLOR, nc);

		m_D3dDevice->SetTexture(stage, m_Texture);
}

void FileTexture::DeActivateTexture(DWORD stage)
{
	//
	// Is this the right way to deactivate texture?
	//
	m_D3dDevice->SetTexture(stage, NULL);

	//
	// I think this is a better way to disable texture?
	m_D3dDevice->SetTextureStageState( stage, D3DTSS_COLOROP,   D3DTOP_DISABLE);

  // Disable Texture coordinate transform
	m_D3dDevice->SetTextureStageState(0 /* Texture Stage */, D3DTSS_TEXTURETRANSFORMFLAGS, 
								D3DTTFF_DISABLE /* Says Do not transform any more */);
}
