// UW Bothell Graphics Library Copyright (c) 2004 University of Washington Bothell
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
/////////////////////////////////////////////////////////////////////////////////////////
#pragma once

#include "uwbgl_D3DDefines.h"

namespace UWBGL_D3D
{
  // Container for the D3D connection and active device.
  // This is a "singleton" class. Only a single instance of this class ever exists.
  // Notice that the constructor is private.

  class FileTexture;
  typedef std::vector<FileTexture*> FileTextureTable;

  class GraphicsSystem
  {
  public:
    static bool Initialize();
    static void Terminate();
    static GraphicsSystem& GetSystem(){ return m_Graphics;}
  private:
    static GraphicsSystem m_Graphics;

  public:
    LPDIRECT3D9 m_pD3D; // Abstract connection to D3D output pipeline

    // returns if GraphicsSystem has already been initialized
    // this is handy funciton to call to determine if system 
    // initialization has happen yet.
    bool GraphicsIsInitialized();

    void SetActiveDevice(LPDIRECT3DDEVICE9 pDevice);
    void SetActiveResourceTable(ResourceTable *pTable) { m_pActiveResourceTable = pTable; }

    LPDIRECT3DDEVICE9 GetActiveDevice();
    ResourceTable* GetActiveResourceTable() { return m_pActiveResourceTable; }
    ID3DXMatrixStack* GetWorldStack() {return m_pObj2WorldStack;}

    virtual ~GraphicsSystem();
  private:
    GraphicsSystem();

    // The D3D device that is actively being drawn to. Since the UWBGL system is a single
    // threader application, this works great. If the library were switched to support
    // multi-threaded apps. with multiple devices, a different implementation would be required.
    LPDIRECT3DDEVICE9 m_pActiveDevice;
    ResourceTable*    m_pActiveResourceTable;

    // We will use this stack to push/pop matrix to support object
    // to world coordinate transformations
    ID3DXMatrixStack* m_pObj2WorldStack;
  };
};
