#pragma once

#include "uwbgl_D3DDefines.h"

namespace UWBGL_D3D
{
    //
    //
    // 
    class InteractiveMesh {

      protected:
        static const int kInvalidVertexIndex;
    
        LPDIRECT3DDEVICE9  m_Device;
                  // for which drawable this Mesh is created for
    
        bool    m_DrawNormal;
        bool    m_DrawControlGrid;

        //
        //  D3DXMESH: in our case:
        //
        //    1. GetVertexBuffer() returns a pointer to TmyVertex informaiton
        //    2. GetIndexBuffer() returns a pointer to seqeuence of unsigned short
        //    3. in a group defining each face of all the triangles defeind.
        LPD3DXMESH    	m_Mesh;
    
        //
        // NumControlVertices is num on each size, e.g. input n means we have a nxn mesh
        //
        // NumVertices = NumControlVertices * NumControlVertices
        // NumFaces = (NumControlVertices - 1)^2 * 2  // (numControlVertices-1) squared then multiply 2
        //
        int    m_NumControlVertices;
        int    m_NumFaces;
        int    m_NumVertices;
    
        int      		 m_SelectedVertexIndex;
        D3DXVECTOR3  m_SelectedVertex;    // this is only valid when SelectedVertex is valid
                          								// when valid, this value is copied to the buffer each time we draw
    
    
        float  m_VertexDistance;
    
        float ControlVertexSize() const {
                		return VertexDistance() * 0.2f;
              }
    
    
        int  IndexFromRowColumn(int r, int c) const {
                return r * m_NumControlVertices + c;
        }
    
        void LoadSelectedVertexFromD3D();
        void SetSelectedVertexToD3D();
    
        //
        // 
        virtual void ComputeSurface();


        void  DrawMeshControlGrid();
        void  DrawNormalVectors();
    
      public:
    
        int   Resolution() const { return m_NumControlVertices; }
        float VertexDistance() const { return m_VertexDistance; }
    
        void  SetToNoSelectedVertex();
        void  SelectVertex(int r, int c);
        void  SelectedVertexPosition(D3DXVECTOR3 &p) const;
        void  SetSelectedVertexPosition(const D3DXVECTOR3 &p);
    
        void  DrawMesh(bool useColor, const D3DXCOLOR &c);

        bool  DrawNormalIsEnabled() { return m_DrawNormal; }
        void  SetDrawNormalTo(bool r) { m_DrawNormal = r; }
        void  EnableDrawNormal() { m_DrawNormal = true; }
        void  DisableDrawNormal() { m_DrawNormal = false; }

        bool DrawControlGridIsEnabled() { return m_DrawControlGrid; }
        void SetDrawControlGridTo(bool r) { m_DrawControlGrid = r; }
        void EnableDrawControlGrid() { m_DrawControlGrid = true; }
        void DisableDrawControlGrid() { m_DrawControlGrid = false; }
    
        InteractiveMesh(LPDIRECT3DDEVICE9, int res=20, char *outFileName = NULL);
        ~InteractiveMesh();
    };
};
