#include "uwbgl_D3DInteractiveMesh.h"
#include "uwbgl_D3DDeviceVertexFormat.h"
#include "uwbgl_D3DLinePrimitive.h"
#include "uwbgl_D3DPointPrimitive.h"

using namespace UWBGL_D3D;

//
// 
void
InteractiveMesh::DrawMesh(bool useColor, const D3DXCOLOR &c)
{

	//
	// 1. make sure we load the new value of the selected vertex into the hardware ...
	//
	SetSelectedVertexToD3D();

	//
	// 2. Recompute Normal: this is VERY EXPENSIVE, should optimize in real application ...
	D3DXComputeNormals(m_Mesh, NULL);

  
    D3DMATERIAL9 m;
    memset(&m, 0, sizeof(D3DMATERIAL9));
    m.Ambient.a = c.a;
    m.Specular.a = c.a;
    m.Diffuse.a = c.a;
    m.Emissive = c;

    if (useColor) {
        // if lighting is off, use emissive to cheat the correct color ...
        m_Device->SetRenderState(D3DRS_LIGHTING, TRUE);
        // Mesh comes with material, so we should always be doing lighting
        // Set the material and texture for this subset, either with the passed in color, or with the original mesh color
         m_Device->SetMaterial(&m);
    } 
    

	m_Mesh->DrawSubset(0);


  if (DrawControlGridIsEnabled())
        DrawMeshControlGrid();

  if (DrawNormalIsEnabled())
        DrawNormalVectors();

}

void 
InteractiveMesh::DrawNormalVectors()
{
	LinePrimitive l;
	l.DisableLighting();
	l.SetDrawColor(D3DXCOLOR(1.0f, 1.0f, 1.0f, 1.0f));

	void *pVBuffer;
	m_Mesh->LockVertexBuffer( 0, &pVBuffer );
			DeviceVertexFormat *v = (DeviceVertexFormat *)pVBuffer;
			for (int i=0; i<m_NumVertices; i++) {
					D3DXVECTOR3 p = v[i].m_Point + (v[i].m_Normal * 0.1f);
					l.SetEnds(v[i].m_Point.x, v[i].m_Point.y, v[i].m_Point.z, p.x, p.y, p.z);
					l.DrawPrimitive();
		}
	m_Mesh->UnlockVertexBuffer();
}


void
InteractiveMesh::DrawMeshControlGrid()
{
	PointPrimitive sph;
	sph.SetPointSize(4);
	sph.DisableLighting();
	sph.SetDrawColor(D3DXCOLOR(0.8f, 0.8f, 0.8f, 1.0f));

	void *pVBuffer;
	m_Mesh->LockVertexBuffer( 0, &pVBuffer );
		DeviceVertexFormat *v = (DeviceVertexFormat *)pVBuffer;
		for (int i=0; i<m_NumVertices; i++) {
				sph.SetFirstVertex(D3DXVECTOR3(v[i].m_Point.x, v[i].m_Point.y, v[i].m_Point.z));
				sph.DrawPrimitive();
		}
	m_Mesh->UnlockVertexBuffer();

	//
	// Now draw the selected vertex ...
	if (m_SelectedVertexIndex != kInvalidVertexIndex) {
		sph.SetDrawColor(D3DXCOLOR(1.0f, 0.0f, 0.0f, 1.0f));
		sph.SetPointSize(8);
		sph.SetFirstVertex(D3DXVECTOR3(m_SelectedVertex.x, m_SelectedVertex.y, m_SelectedVertex.z));
		sph.DrawPrimitive();
	}

}
