#include "uwbgl_D3DInteractiveMesh.h"
#include "uwbgl_D3DDeviceVertexFormat.h"


using namespace UWBGL_D3D;

//
// Selection of control vertex needs a little care since we
// are caching a copy of the vertex geometry locally in main
// memory, we must make sure the cached copy is consistent with
// the version inside the D3D pipeline ...
//


void InteractiveMesh::LoadSelectedVertexFromD3D()
{
	if (m_SelectedVertexIndex != kInvalidVertexIndex) {
		void *pVBuffer;
		m_Mesh->LockVertexBuffer( 0, &pVBuffer );
				DeviceVertexFormat *pCurrentV = &(((DeviceVertexFormat *)pVBuffer)[m_SelectedVertexIndex]);
				m_SelectedVertex = pCurrentV->m_Point;
		m_Mesh->UnlockVertexBuffer();
	}
}

void InteractiveMesh::SetSelectedVertexToD3D()
{
	if (m_SelectedVertexIndex != kInvalidVertexIndex) {
		void *pVBuffer;
		m_Mesh->LockVertexBuffer( 0, &pVBuffer );
				DeviceVertexFormat *pCurrentV = &(((DeviceVertexFormat *)pVBuffer)[m_SelectedVertexIndex]);
				pCurrentV->m_Point = m_SelectedVertex;
		m_Mesh->UnlockVertexBuffer();
	}
}

void InteractiveMesh::SetToNoSelectedVertex() 
{
	//
	// check if we are currently selecting a valid vertex
	if (m_SelectedVertexIndex != kInvalidVertexIndex) {
		// now we should update the index into the hardware ...
		SetSelectedVertexToD3D();
	}
	m_SelectedVertexIndex = kInvalidVertexIndex;
}


void InteractiveMesh::SelectVertex(int r, int c)
{
	//
	// 1. make sure we are not currently caching anything ...
	//
	SetSelectedVertexToD3D();

	m_SelectedVertexIndex = IndexFromRowColumn(r, c);

	LoadSelectedVertexFromD3D();
}

void InteractiveMesh::SelectedVertexPosition(D3DXVECTOR3 &p) const
{
	p = m_SelectedVertex;
}

void InteractiveMesh::SetSelectedVertexPosition(const D3DXVECTOR3 &p)
{
	m_SelectedVertex = p;
	//
	// Notice we cache this, and only set into the Hardware before we draw ...
	//
}