#include "uwbgl_D3DLight.h"
#include "uwbgl_D3DGraphicsSystem.h"

using namespace UWBGL_D3D;

Light::Light(int id)
:   m_LightID(id)
  , m_bLightIsOn(false)
  , m_bDrawLight(false)
{
  // Nice set of default values ...
  SetDiffuse(0.8f);
  SetSpecular(0.2f);
  SetAmbient(0.1f);

  SetRange(200.0f);
  SetFalloff(1.0f);
  SetAttenuation0(1.0f);
  SetAttenuation1(0.0f);
  SetAttenuation2(0.0f);
  SetTheta(M_PI/4.0f);		// Spot light inner cone radius
  SetPhi(M_PI/3.0f);			// Spot light outter cone radius

  SetPosition(0.0f, 0.0f, 0.0f);
  SetDirection(0.0f, -1.0f, 0.0f);
}

Light::~Light()
{
}

void Light::DrawLight() {
  if (LightIsSwitchedOn() && ShouldDrawLight() )
    DrawLightGeometry();
}

void Light::SwitchOnLight()
{
  m_bLightIsOn = true;
}

void Light::SwitchOffLight()
{
  m_bLightIsOn = false;
}

void Light::SetLightToDevice() const
{
  LPDIRECT3DDEVICE9 device = GraphicsSystem::GetSystem().GetActiveDevice();
  if(NULL == device || m_LightID == -1)
    return;

  if ( LightIsSwitchedOn() ) {
      device->SetLight(m_LightID, &m_d3dLight);
      device->LightEnable(m_LightID,TRUE);
  } else {
      device->LightEnable(m_LightID,FALSE);
  }
}


void Light::SetPosition(float x, float y, float z)
{
  m_d3dLight.Position.x = x;
  m_d3dLight.Position.y = y;
  m_d3dLight.Position.z = z;
}

void Light::SetPosition(const D3DXVECTOR3& position)
{
  m_d3dLight.Position = position;
}

void Light::GetPosition(D3DXVECTOR3& position) const
{
  position = m_d3dLight.Position;
}


void Light::SetDirection(float x, float y, float z)
{
  m_d3dLight.Direction.x = x;
  m_d3dLight.Direction.y = y;
  m_d3dLight.Direction.z = z;
}

void Light::SetDirection(const D3DXVECTOR3& d)
{
  m_d3dLight.Direction = d;
}

void Light::GetDirection(D3DXVECTOR3& direction) const
{
  direction = m_d3dLight.Direction;
}