#pragma once

#include "uwbgl_D3DDefines.h"

#define SetMaterialProperty(materialType) 								\
	void Set##materialType##(float v)	{								\
		m_Material.##materialType##.r = v;										\
		m_Material.##materialType##.g = v;										\
		m_Material.##materialType##.b = v;										\
		m_Material.##materialType##.a = v;										\
	}																	\
	void Set##materialType##(float r, float g, float b, float a=1.0) {	\
		m_Material.##materialType##.r = r;										\
		m_Material.##materialType##.g = g;										\
		m_Material.##materialType##.b = b;										\
		m_Material.##materialType##.a = a;										\
	}																	\
	void Set##materialType##(const D3DXCOLOR &v) {						\
		m_Material.##materialType##.r = v.r;									\
		m_Material.##materialType##.g = v.g;									\
		m_Material.##materialType##.b = v.b;									\
		m_Material.##materialType##.a = v.a;									\
	}																	\
	void Get##materialType##(D3DXCOLOR &v) const {						\
		v.r = m_Material.##materialType##.r;									\
		v.g = m_Material.##materialType##.g;									\
		v.b = m_Material.##materialType##.b;									\
		v.a = m_Material.##materialType##.a;									\
	}																	\
	float Get##materialType##R() const { return m_Material.##materialType.r;}	\
	float Get##materialType##G() const { return m_Material.##materialType.g;}	\
	float Get##materialType##B() const { return m_Material.##materialType.b;}	\
	float Get##materialType##A() const { return m_Material.##materialType.a;}


//
// material property for D3DX: We wrap around D3DMATERIAL9 structure providing a collection
// of accessing methods.
//
namespace UWBGL_D3D
{
	class Material {
		//
		private:
			D3DMATERIAL9	m_Material;		// this is the material
										// has: 	Diffuse		(D3DCOLORVALUE)
										// 				Ambient		(D3DCOLORVALUE)
										// 				Specular	(D3DCOLORVALUE)
										// 				Emissive	(D3DCOLORVALUE)
										// 				power 		(float)
	
		public:
			Material() {
				//
				//
				// nice shade of Husky purple ...
				//
				SetAmbient(0.13f, 0.065f, 0.195f, 1.0f);
				//
				//
				// grey-ish  dull color otherwise ...
				// //
				SetEmissive(0.1f);
				SetDiffuse(0.8f);
				SetSpecular(0.2f);
				SetPower(1.0f);
			}
	
			//
			// define the setting and accessing functions
			//
			SetMaterialProperty(Emissive)
			SetMaterialProperty(Ambient)
			SetMaterialProperty(Diffuse)
			SetMaterialProperty(Specular)
	
			void  SetPower(float s) { m_Material.Power = s; }
			float GetPower() { return m_Material.Power; }
	
      const D3DMATERIAL9* GetD3DMaterial() const { return &m_Material; }

	};
}
