#include "uwbgl_D3DGraphicsSystem.h"
#include "uwbgl_D3DDeviceVertexFormat.h"
#include "uwbgl_D3DRectanglePrimitive.h"
#include "uwbgl_D3DCollider.h"

using namespace UWBGL_D3D;

RectanglePrimitive::RectanglePrimitive() 
{
}

RectanglePrimitive::~RectanglePrimitive() 
{}

void RectanglePrimitive::SetFirstVertex(const D3DXVECTOR3 &p)
{
  DrawablePrimitive::SetFirstVertex(p);
  if (NumVertices() > 1) ReplaceVertexAt(1, p);
}

void RectanglePrimitive::AppendVertex(const D3DXVECTOR3 &p)
{
	if (NumVertices() < 2)
		DrawablePrimitive::AppendVertex(p);
	else
		ReplaceVertexAt(1, p);
}

void RectanglePrimitive::MovePrimitiveTo(const D3DXVECTOR3 &pt)
{
  if (NumVertices() == 0) return;

  if (NumVertices() == 1)
  {
		m_Vertices[0] = pt;
		return;
	}

  D3DXVECTOR3 d = pt - m_Vertices[0];
	m_Vertices[1] += d;
  m_Vertices[0] = pt;
}
void RectanglePrimitive::GetColliderList(ColliderList &list)
{
  if (NumVertices() < 2) return;

  D3DXVECTOR3 v = m_Vertices[0] - m_Vertices[1];
  Collider c(m_Vertices[0], D3DXVec3Length(&v));
  list.Append(c);
}

void RectanglePrimitive::DrawPrimitiveGeometry()
{
  if (NumVertices() < 2) return;

  LPDIRECT3DDEVICE9 d3dDevice = GraphicsSystem::GetSystem().GetActiveDevice();
  if (d3dDevice == NULL) return;

	// We are going to use TRIANGLESTRIP for drawing of rectangle
	// we need the 4 positions defined as:

	//   Position 1 is the first corner
	//            2 is the X-change corner
	//            3 is the Y-change corner
	//            4 is the X/Y both change corner
	DeviceVertexFormat v[4];
		
	D3DXVECTOR3 delta = m_Vertices[1] - m_Vertices[0];
	delta.x = fabs(delta.x);
	delta.y = fabs(delta.y);
	delta.z = fabs(delta.z);

  for (int i=0; i<4; i++)
  {
		v[i].m_Point = m_Vertices[0];
    v[i].m_Normal = D3DXVECTOR3(0,1,0);
		v[i].m_Color = D3DCOLOR_COLORVALUE(m_DrawColor.r, m_DrawColor.g, m_DrawColor.b, m_DrawColor.a);
		v[i].m_U = v[i].m_V = v[i].m_U1 = v[i].m_V1 = 0.0f;
	}
	v[3].m_Color = D3DCOLOR_COLORVALUE(0.9f, 0.9f, 0.9f, 0.9f);
	v[0].m_Point -= delta;
	v[0].m_U = v[0].m_U1 = 0.0f;
	v[0].m_V = v[0].m_V1 = 1.0f;

	v[1].m_Point.x += delta.x;
	v[1].m_Point.y -= delta.y;
	v[1].m_Point.z -= delta.z;
	v[1].m_U = v[1].m_U1 = 1.0f;
	v[1].m_V = v[1].m_V1 = 1.0f;

	v[2].m_Point.x -= delta.x;
  	v[2].m_Point.y += delta.y;
	v[2].m_Point.z += delta.z;
	v[2].m_U = v[2].m_U1 = 0.0f;
	v[2].m_V = v[2].m_V1 = 0.0f;

	v[3].m_Point += delta;
	v[3].m_U = v[3].m_U1 = 1.0f;
	v[3].m_V = v[3].m_V1 = 0.0f;

	d3dDevice->DrawPrimitiveUP(D3DPT_TRIANGLESTRIP, 2, (CONST void *)v, sizeof(DeviceVertexFormat));
}