#include "uwbgl_D3DSceneNode.h"
#include "uwbgl_D3DGraphicsSystem.h"
#include "uwbgl_D3DMeshPrimitive.h"
#include "uwbgl_D3DAxisFrameSceneNode.h"

using namespace UWBGL_D3D;

SceneNode::SceneNode(const char* name)
: m_pPrimitive(NULL)
, m_Velocity(0,0,0)
, m_pParentNode(NULL)
, m_bShowAxisFrame(false)
, m_bShowPivot(false)
, m_Name(name)
{
}

SceneNode::~SceneNode()
{
  if(m_pPrimitive) delete m_pPrimitive;
  DestroyChildNodes();
}

void SceneNode::DrawNode()
{
  LPDIRECT3DDEVICE9 d3dDevice = GraphicsSystem::GetSystem().GetActiveDevice();
  ID3DXMatrixStack* pStack =  GraphicsSystem::GetSystem().GetWorldStack();
  
  if (  ShowPivotIsEnabled() || ShowAxisFrameIsEnabled() )
  {
      D3DXVECTOR3 p = m_XForm.XFormPivot() + m_XForm.Translation();
      pStack->Push();
      {
        if (ShowPivotIsEnabled() ) {  
            pStack->Push();
                pStack->TranslateLocal(p.x, p.y, p.z);       
                pStack->ScaleLocal(0.5f, 0.5f, 0.5f);
                d3dDevice->SetTransform(D3DTS_WORLD,pStack->GetTop());
          
                MeshPrimitive m(eSphereMesh);
                m.SetDrawColor(D3DXCOLOR(0.0f, 1.0f, 0.0f, 0.0f));
                m.SetFillMode(D3DFILL_WIREFRAME);
                m.DrawPrimitive();
            pStack->Pop();
          }

          if (ShowAxisFrameIsEnabled()) {
              D3DXMATRIX m;
              m_XForm.GetOrientation(m);

              pStack->Push();
                  pStack->TranslateLocal(p.x, p.y, p.z);
                  d3dDevice->SetTransform(D3DTS_WORLD,pStack->GetTop());
                  AxisFrameSceneNode a;
                  a.m_XForm.SetOrientation(m);
                  a.m_XForm.SetScaleTo(1.2f);
                  a.DrawNode();
              pStack->Pop();
          }
      }
      pStack->Pop();
  }

  pStack->Push();
  {
    m_XForm.SetUpD3DXformStack(pStack);
    d3dDevice->SetTransform(D3DTS_WORLD,pStack->GetTop());

       

    if(m_pPrimitive) m_pPrimitive->DrawPrimitive();
    for(unsigned int i=0; i<m_ChildNodes.size(); i++) m_ChildNodes[i]->DrawNode();
  }
  pStack->Pop();
  // Not only do we need to pop off our content from the stack, we have to 
  // make sure we re-load the WORLD matrix with whatever it used to contain
  d3dDevice->SetTransform(D3DTS_WORLD, pStack->GetTop());
}

void SceneNode::SetVelocity(const D3DXVECTOR3 &v)
{
  m_Velocity = v;
}

const D3DXVECTOR3& SceneNode::GetVelocity() const
{
  return m_Velocity;
}

void SceneNode::MoveNodeByVelocity()
{
  m_XForm.UpdateTranslationBy(m_Velocity);
}

void SceneNode::SetPrimitive(PrimitiveBase* pPrimitive)
{
  if(m_pPrimitive) delete m_pPrimitive;
  m_pPrimitive = pPrimitive;
  if(m_pPrimitive) m_pPrimitive->SetParentNode(this);
}

SceneNode* SceneNode::AppendChildNode(const char* name)
{
  SceneNode* pNode = new SceneNode(name);
  InsertChildNode(pNode);
  return pNode;
}

void SceneNode::InsertChildNode(SceneNode* pChild)
{
  if(NULL == pChild) return;
  pChild->m_pParentNode = this;
  m_ChildNodes.push_back(pChild);
}

void SceneNode::DestroyChildNodes()
{
  //recursively destroy the tree
  for(unsigned int i=0; i<m_ChildNodes.size(); i++)
  {
    delete m_ChildNodes[i];
  }
  m_ChildNodes.clear();
}

SceneNode* SceneNode::GetChildNode(unsigned int i)
{
  if(i>=m_ChildNodes.size()) return NULL;
  return m_ChildNodes[i];
}
const SceneNode* SceneNode::GetChildNode(unsigned int i) const
{
  if(i>=m_ChildNodes.size()) return NULL;
  return m_ChildNodes[i];
}