// UW Bothell Graphics Library Copyright (c) 2004 University of Washington Bothell
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
/////////////////////////////////////////////////////////////////////////////////////////
#pragma once

#include "uwbgl_D3DXFormInfo.h"
#include "uwbgl_D3DPrimitiveBase.h"
#include <vector>
#include <string>

namespace UWBGL_D3D
{
  class SceneNode
  {
  public:
    SceneNode(const char* name = "SceneNode");
    virtual ~SceneNode();

    void DrawNode();
    void SetPrimitive(PrimitiveBase* pPrimitive);
    PrimitiveBase* GetPrimitive(){return m_pPrimitive;}
    const PrimitiveBase* GetPrimitive() const{return m_pPrimitive;}

    SceneNode* AppendChildNode(const char* name = "SceneNode");
    void InsertChildNode(SceneNode* pChild);
    unsigned int NumChildren() const {return (unsigned int)m_ChildNodes.size();}
    void DestroyChildNodes();
    SceneNode* GetChildNode(unsigned int i);
    const SceneNode* GetChildNode(unsigned int i) const;

    void MoveNodeByVelocity();
    void SetVelocity(const D3DXVECTOR3 &v);
    const D3DXVECTOR3& GetVelocity() const;

    void SetName(const char* name) {m_Name = name;}
    virtual const char* GetName() const { return m_Name.c_str(); }
   

    // Show Pivot control
    void EnableShowPivot() { m_bShowPivot = true; }
    void DisableShowPivot() { m_bShowPivot = false; }
    void SetShowPivot(bool b) { m_bShowPivot = b; }
    bool ShowPivotIsEnabled() const { return m_bShowPivot; }


		// Show AxisFrmae control
    void EnableShowAxisFrame() { m_bShowAxisFrame = true; }
    void DisableShowAxisFrame() { m_bShowAxisFrame = false; }
    void SetShowAxisFrame(bool b) { m_bShowAxisFrame = b; }
    bool ShowAxisFrameIsEnabled() const { return m_bShowAxisFrame; }

    XFormInfo m_XForm;
  protected:
    //A node can contain primitive. Each primitive item can only have one node parent. The node
    //ALWAYS controls the lifetime of it's internal primitive. This means all primitive must be
    //declared on the heap!!
    PrimitiveBase* m_pPrimitive;
    std::vector<SceneNode*> m_ChildNodes;
    D3DXVECTOR3 m_Velocity;
    bool        m_bShowPivot;
    SceneNode* m_pParentNode; // knowing the parent node allows us to determine full transformation
    bool m_bShowAxisFrame; // Should a frame be drawn at scenenode's local origin?

    std::string m_Name;
  };
};