#pragma once
#include "uwbgl_Defines.h"

/// To work with a 3D graphics API:
///
/// 1. We must establish a "connection" to the GraphicsAPI (hardware)
///    this functionality is supported by the GraphicsSystem.cpp class
///
/// 2. We must create a "graphics device" for this opened connection 
///    (it is possible to create multiple devices for each connection)
///    Each "graphics device" is typically tied tightly to the actual 
///    draw area on the screen. So ...
///
/// 3. If the draw area is changed, one typically needs to inform the "graphics device"
///    of the actual draw area size change
///
/// 4. The graphics device is global to all drawings made to the actual area, so
///    typically, before each drawing, one needs to "initialize the graphics device"
///    and after drawing one should "restore the graphics device"


/// The IViewport struct acts as a pure abstract base class for API specific viewports. This
/// is done because this is the one item that must be shared across our different libraries.
/// By having absolutely no implementation, no project dependencies are created.
struct IViewport
{
  virtual ~IViewport(){}

  virtual bool GraphicsDeviceReady() = 0;

  // Create and Initialize graphics drawing context
  virtual bool CreateGraphicsDevice(HWND OutputDrawArea)=0;
  virtual void ResizeGraphicsDevice(int width, int height)=0;

  virtual void InitializeDeviceForDrawing()=0;
  virtual void DrawToDevice()=0;
  virtual void RestoreDeviceAfterDrawing()=0;


  //mouse events
  virtual void OnMouseButton(bool down, unsigned int nFlags, int hardwareX, int hardwareY)=0;
  virtual void OnMouseMove(unsigned int nFlags, int hardwareX, int hardwareY)=0;
};