#include "uwbgl_ViewportBase.h"

ViewportBase::ViewportBase(): m_width(-1), m_height(-1)
{
}

ViewportBase::~ViewportBase()
{
}

bool ViewportBase::CreateGraphicsDevice(HWND hWindow)
{
  //check that the parameter is an actual window
  if(!::IsWindow(hWindow)) return false;

  //set up our initial width and height by getting the parent window size
  RECT rect;
  ::GetWindowRect(hWindow,&rect);
  m_width = rect.right - rect.left;
  m_height = rect.bottom - rect.top;

  m_hAttachedWindow = hWindow;
  return true;
}

void ViewportBase::ResizeGraphicsDevice(int width, int height)
{
  //default implementation of a viewport is sized to cover entire parent window
  m_width = width;
  m_height = height;
}

bool ViewportBase::HardwareToDevice(int hardwareX, int hardwareY, int& deviceX, int& deviceY) const
{
  // the transform is simply a flip of y
  if(m_width<0 || m_height<0) return false;
  deviceX = hardwareX;
  deviceY = m_height - hardwareY;
  return true;
}

void ViewportBase::GetDeviceSize(int& width, int& height) const
{
  width = m_width;
  height = m_height;
}

float ViewportBase::GetAspectRatio() const
{
  if(m_height == 0) return 0.0f;
  return (float)m_width / (float)m_height;
}