#pragma once
#include "uwbgl_IViewport.h"
#include "uwbgl_Defines.h"

/// Common graphics base class for viewports. Contains supporting implementation for resizing
/// and transforming points between hardware and device coordinates.
class ViewportBase : public IViewport
{
public:
  ViewportBase();
  virtual ~ViewportBase();

  //Implement our interface functions
  virtual bool CreateGraphicsDevice(HWND OutputDrawArea);
  virtual void ResizeGraphicsDevice(int width, int height);
  virtual void OnMouseButton(bool down, unsigned int nFlags, int hardwareX, int hardwareY){}
  virtual void OnMouseMove(unsigned int nFlags, int hardwareX, int hardwareY){}

  /// Point transformation from hardware coordinate to device coordinate.
  /// \param hardwareX, hardwareY [in] Hardware coordinate to transform
  /// \param deviceX, deviceY [out] Resulting device coordinate
  // return true if successful
  bool HardwareToDevice(int hardwareX, int hardwareY, int& deviceX, int& deviceY) const;

  /// Get size of viewport in hardware coordinates
  /// \param width, height [out] pixel size of viewport
  void GetDeviceSize(int &width, int& height) const;

  float GetAspectRatio() const;
protected:
  /// Handle to window that this viewport is drawn on
  HWND m_hAttachedWindow;

  /// Width and Height of the viewport in hardware units. Default implementation is the
  /// same size as the parent window (this is not necessary.)
  int m_width;
  int m_height;
};