// UW Bothell Graphics Library Copyright (c) 2004 University of Washington Bothell
// Authors: Kelvin Sung, Steve Baer
// The accompanying library supports CSS Graphics courses taught at UW-Bothell
// See: http://courses.washington.edu/css450/
//      http://courses.washington.edu/css451/
/////////////////////////////////////////////////////////////////////////////////////////

/// uwbgl_MFCSliderCtrlWithEcho.h: header file for a CSliderCtrl-derived slider object
///		with associated text box for UI output
#pragma once
namespace UWBGL_MFC
{
  class SliderCtrlWithEcho;
};


namespace UWBGL_MFC
{
  /// Provides a specialized slider control object that contains a text box.
  ///  - Echos slider value to a static text
  ///  - Support floating point slider range (with resolution of 100K)
  /// 
  /// To use: 
  ///	- Draw a slider on a dialog and create a member variable for the control
  ///	- Replace the CSliderCtrl variable type with SliderCtrlWithEcho
  class SliderCtrlWithEcho : public CSliderCtrl
  {
    DECLARE_DYNAMIC(SliderCtrlWithEcho)

  public:
    SliderCtrlWithEcho();
    virtual ~SliderCtrlWithEcho();

    /// Creates the slider control and text box, determines horizontal or vertical 
    /// orientation, and sets local class data member values.
    /// \param min Minimum value of slider control range
    /// \param max Maximum value of slider control range
    /// \param init Initial value of slider control, defaults to 0.00 if the value of init 
    ///        falls outside the specified range min < init < max
    void Initialize(float min, float max, float init);

    /// Set the value of the slider and the slider position
    /// \return true on successful value update
    bool SetSliderValue(float userValue);

    /// Retrieve the current float value of the slider
    /// \return current float value
    float GetSliderValue();

  protected:
    DECLARE_MESSAGE_MAP()
  	
    /// Converts from user-specified value range to MFC resolution - 100K
    /// \param userValue Floating point value to convert to MFC range
    /// \return int value in MFC resolution
    int UserToMFCPos(float userValue);

    /// Converts from MFC resolution - 100K - to user-specified value range
    /// \param mfcValue Floating point value to convert to user-specified range
    /// \return float value in user-specified range
    float MFCToUserPos(int mfcValue);

    /// Set format for UI output and update UI based on current slider value
    /// \return void; no return type specified
    void UpdateSliderEcho();

    /// Class data members
    /// kMaxSliderRange - const value setting internal, MFC resolution for possible slider values
    static const int kMaxSliderRange = 100000;

    /// minimum range number, specified by user; defaults to 0.00
    float fMinRange;    

    /// total range of slider values, specified by user; defaults to 1.00
    float fTotalRange; 

    /// CStatic object used for UI text box
    CStatic m_MessageWnd;

    /// flag to communicate whether slider bar is vertical or horizontal
    bool m_bIsVert;

  public:
    /// Handler for windows WM_HSCROLL event. Converts slider value and updates member variable
    /// \return void; afx_msg resolves to whitespace after preprocessing
    afx_msg void HScroll(UINT nSBCode, UINT nPos);
    /// Handler for windows WM_VSCROLL event. Converts slider value and updates member variable
    /// \return void; afx_msg resolves to whitespace after preprocessing
    afx_msg void VScroll(UINT nSBCode, UINT nPos);
    /// Handler for windows WM_SHOWWINDOW event. Used to Show or Hide window element
    /// \return void; afx_msg resolves to whitespace after preprocessing
    afx_msg void OnShowWindow(BOOL bShow, UINT nStatus);
  };
};