#include "StdAfx.h"
#include "World.h"

#ifdef _DEBUG
  #define new DEBUG_NEW
#endif

World theWorld;

World::World()
: m_bMoveTiger(false)
,m_PointLight(1)
,m_DirectionalLight(2)
,m_SpotLight(3)
,m_FogColor(1.0f, 1.0f, 1.0f, 1.0f)
,m_FogStart(10.0f)
,m_FogEnd(20.0f)
,m_FogDensity(0.05f)
,m_FogEnabled(false)
,m_FogMode(D3DFOG_LINEAR)
{
  m_RootNode.SetName("Root");
  m_pTigerNode = m_RootNode.AppendChildNode("Tiger");
  m_pCircleNode = m_pTigerNode->AppendChildNode("Circle");

  m_pCraftNode = m_RootNode.AppendChildNode("Craft List");
  m_pFloorNode = m_RootNode.AppendChildNode("Floor20x20");
  m_pTeapotNode = m_RootNode.AppendChildNode("Teapot");

  //Add the tiger mesh
  MeshPrimitive* pMesh = new MeshPrimitive(eTigerMesh);
  pMesh->SetDrawColor(D3DXCOLOR(0.7f, 0.8f, 0.8f, 1.0f));
	pMesh->EnableTexture();
  m_pTigerNode->SetPrimitive(pMesh);
  
  //Add the circle primitive
  m_pCircleNode->SetPrimitive(new CirclePrimitive(1.0f));

  // Add the floor
  MeshPrimitive *floor = new InteractiveMeshPrimitive(201);
  m_pFloorNode->SetPrimitive(floor);
  

  //Add the craft primitive into a primitive list ...
  DrawablePrimitiveList *list = new DrawablePrimitiveList();
  m_pCraftNode->SetPrimitive(list);
  pMesh = new MeshPrimitive(eShusuiMesh);
  list->InsertDrawablePrimitive(pMesh);
  CirclePrimitive *c = new CirclePrimitive(D3DXVECTOR3(0.0f, 1.0f, 0.0f), 1.0f);
  c->SetDrawColor(D3DXCOLOR(0.8f, 0.9f, 0.7f, 1.0f));
  list->InsertDrawablePrimitive(c);

  // Add the teapot primitive
  pMesh = new MeshPrimitive(eTeapotMesh);
  m_pTeapotNode->SetPrimitive(pMesh);

  Reset();
}

void World::Reset()
{
  m_bMoveTiger = false;
  m_pTigerNode->m_XForm.SetScaleTo(2,2,2);
  m_pTigerNode->m_XForm.SetTranslationTo(0,0,0);

  m_pCraftNode->m_XForm.SetTranslationTo(2,2,2);
  m_pCraftNode->m_XForm.SetRotationToRadian(0,0,0);
  m_pCraftNode->m_XForm.SetScaleTo(1,1,1);

  m_pCircleNode->m_XForm.SetTranslationTo(-2,0,-2);
  m_pCircleNode->m_XForm.SetRotationToRadian(0,0,0);
  m_pCircleNode->m_XForm.SetScaleTo(1,1,1);

  m_pTeapotNode->m_XForm.SetTranslationTo(-2.0f, 0.0f, 2.0f);
  
  m_pFloorNode->m_XForm.SetTranslationTo(0.0f, 0.0f, 0.0f);
  m_pFloorNode->m_XForm.SetScaleTo(6.0f);

  m_PointLight.SwitchOnLight();
  m_PointLight.SetPosition(2.0f, 5.0f, 2.0f);
  m_PointLight.SetDiffuse(0.2f);

  m_DirectionalLight.SwitchOnLight();
  m_DirectionalLight.SetPosition(0.0f, 3.0f, 0.0f);
  m_DirectionalLight.SetDiffuse(0.2f, 0.1f, 0.1f);

  m_SpotLight.SwitchOnLight();
  m_SpotLight.SetPosition(-2.0f, 5.0f, 2.0f);
  m_SpotLight.SetDiffuse(0.2f, 0.2f, 0.5f);
}

SceneNode* World::GetRootNode()
{
  return &m_RootNode;
}

void World::SetMoveTiger(bool on)
{
  m_bMoveTiger = on;
}