#include "StdAfx.h"
#include "World.h"

#ifdef _DEBUG
  #define new DEBUG_NEW
#endif

#define kTigerSpeed 2.0f/40.0f
        // tiger moves at 10 pixels per second.

void World::UpdateWorld()
{
  if(m_bMoveTiger && (NULL != m_pTigerNode) )
  {
    XFormInfo& xt = m_pTigerNode->m_XForm;
    D3DXMATRIX m;
    xt.GetOrientation(m);


    // Now, we know the tiger's "front" is -ve z-axis, so to move tiger
    // forward we will ...
    D3DXVECTOR3 vt(0, 0, -1);
    D3DXVECTOR3 velocity;
    D3DXVec3TransformCoord(&velocity, &vt, &m);
        // The above three lines accomplishes the exact same thing as
        //  velocity.x = -m_.31;
        //  velocity.y = -m_.32;
        //  velocity.z = -m_.33;

        // since m is a rotation matirx, we know the row/column vectors are all normalized
    velocity *= kTigerSpeed;

    xt.UpdateTranslationBy(velocity);
  }


          // ************ DO THE CRAFT *****************************************************
          // The Craft, we know the craft's front is facing the -ve z-axis
          // we want the craft to point towards the tiger ...
          // we want to figure out the rotation from craft's original position to pointing towards the tiger
          D3DXVECTOR3 craftFront(0.0f, 0.0f, -1.0f);

          // 1. figure out the vector from the craft towards the tiger.
          D3DXVECTOR3 toTiger = m_pTigerNode->m_XForm.Translation() - m_pCraftNode->m_XForm.Translation();
          D3DXVec3Normalize(&toTiger, &toTiger);

          // 2. Now figure out the angle between the craft's front (-ve z-axis) and toTiger-vector
          float cosTheta = D3DXVec3Dot(&toTiger, &craftFront);
          float theta = acosf(cosTheta);


          // 3. Now the axis that is perpendicular to both craftFront and toTiger
          D3DXVECTOR3 rotationAxis;
          D3DXVec3Cross(&rotationAxis, &craftFront, &toTiger);
          D3DXVec3Normalize(&rotationAxis, &rotationAxis);

          // 4. now we know the exact rotation we want the craft to do ...
          D3DXQUATERNION q;
          D3DXQuaternionRotationAxis(&q, &rotationAxis, theta);

          // 5. Now set this rotation to the m_pCraftNode ...
          m_pCraftNode->m_XForm.SetOrientationQ(q);
          



          // ************ NOW: Let's DO THE Teapot *****************************************************
          // Apply the same algorithm, rotate the teapot from its original position towards the moving tiger
          // only this time, let's put some damping in ...
          // Notice the front of the teapot is +ve x-axis ...
          D3DXVECTOR3 teapotFront(1.0f, 0.0f, 0.0f);

          // 1. Figure out the vector from the teapot towards the tiger.
          toTiger = m_pTigerNode->m_XForm.Translation() - m_pTeapotNode->m_XForm.Translation();
          D3DXVec3Normalize(&toTiger, &toTiger);

          // 2. Now figure out the angle between the teapot front (+ve x-axis) and toTiger-vector
          cosTheta = D3DXVec3Dot(&toTiger, &teapotFront);
          theta = acosf(cosTheta);

          // 3. Now the axis that is perpendicular to both teapotFront and toTiger
          D3DXVec3Cross(&rotationAxis, &teapotFront, &toTiger);
          D3DXVec3Normalize(&rotationAxis, &rotationAxis);

          // 4. now we know the exact rotation we want the teapot to do ...
          D3DXQuaternionRotationAxis(&q, &rotationAxis, theta);

          // 5. Now, instead of forcing the teapot to look at the tiger immediately, let's do it gradually ...
          D3DXQUATERNION tq; 
          m_pTeapotNode->m_XForm.GetOrientationQ(tq); // this is the current teapot orientation ...

          // 6. This is the gradual rotation we want to do ...
          D3DXQUATERNION newQ;
          D3DXQuaternionSlerp(&newQ, &tq, &q, 0.05f);
                      // We want to interpolate from tq towards q, with 5% increment ...
                      // newQ is the result

          m_pTeapotNode->m_XForm.SetOrientationQ(newQ);

          if (m_bMoveTiger) {
              // Let the teapot chase the tiger ..
              // remember teapot's front is its X-axis ..
              D3DXMATRIX tm;
              m_pTeapotNode->m_XForm.GetOrientation( tm );
              teapotFront.x = tm._11;
              teapotFront.y = tm._12;
              teapotFront.z = tm._13;
              teapotFront *= (kTigerSpeed * 0.95f);
              m_pTeapotNode->m_XForm.UpdateTranslationBy( teapotFront );
          }
  
}
